<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2021 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

/**
 * Class KoalapayApi
 */
class KoalapayApi
{
    /**
     * Url
     * @var string
     */
    protected $url;

    /**
     * Token
     * @var string
     */
    protected $token;

    /**
     * Requests Hooks
     * @var string
     */
    protected $hooks;

    /**
     * KoalapayApi constructor.
     * @param string $url api url
     * @param string $token api token
     */
    public function __construct($url, $token)
    {
        $this->url = trim($url, '/') . '/';
        $this->token = $token;

        $this->hooks = new Requests_Hooks();
        $this->hooks->register('requests.before_request', array(&$this, 'beforeRequest'));
        $this->hooks->register('requests.after_request', array(&$this, 'afterRequest'));
    }

    /**
     * Adds default headers
     * @param $url
     * @param $headers
     * @param $data
     * @param $type
     * @param $options
     */
    public function beforeRequest(&$url, &$headers, &$data, &$type, &$options)
    {
        $headers['Authorization'] = 'Bearer ' . $this->token;
        if (!isset($headers['Content-type'])) {
            $headers['Content-type'] = 'application/json';
        }
        if (is_string($data) && preg_match('#^{#', trim($data)) && !json_decode($data)) {
            $msg = 'JSON Error ('.json_last_error_msg().') for: "'.$data.'"';
            $msg .= ' | url: ' . $url . ' | type: ' . $type . ' | options: ' . print_r($options, true);
            throw new Exception($msg);
        }
    }

    /**
     * Adds default headers
     * @param $url
     * @param $headers
     * @param $data
     * @param $type
     * @param $options
     */
    public function afterRequest(&$response)
    {
        $response->data = null;
        if (isset($response->body)) {
            $result = json_decode($response->body);
            if ($result) {
                $response->data = $result;
            }
        }
    }

    /**
     * Check if API is alive
     * @return bool|string true if alive or the error message
     */
    public function isAlive()
    {
        try {
            $response = Requests::get($this->url);
        } catch (Requests_Exception $e) {
            return $e->getMessage();
        }
        return (bool)$response;
    }

    /**
     * Get Current customer info
     * @return stdClass current customer info
     *  {
     *      "data": {
     *          "name": "Eve Allen",
     *          "vatNumber": "BE0326598547",
     *          "language": "fr",
     *          "adminUser": {
     *          "firstName": "Marcus",
     *          "lastName": "Leur",
     *          "email": "mlcontact@example.com"
     *      },
     *      "billing": {
     *          "name": "Debbie Upper",
     *          "vatNumber": "BE0326598547",
     *          "address": {
     *              "addressLines": [
     *                  "1007 Mountain Drive",
     *                  "Wayne Manor"
     *              ],
     *              "city": "Gotham",
     *              "postCode": "1000",
     *              "countryCode": "US"
     *          },
     *          "email": "dupcontact@example.com"
     *      },
     *      "subscriptionType": "ideal"
     *  }
     */
    public function getCurrentCustomer()
    {
        $url = $this->url . 'customers/current';
        $response = Requests::get($url, array(), array('hooks' => $this->hooks));
        return $response->data;
    }

    /**
     * Get financing status of the current customer
     * @return stdClass financing status
     * {
     *    "data": {
     *      "eligible": true,
     *      "active": true,
     *      "limit": 25000
     *    }
     * }
     */
    public function getFinancingStatus()
    {
        $url = $this->url . 'customers/current/financing';
        $response = Requests::get($url, array(), array('hooks' => $this->hooks));
        return $response->data;
    }

    /**
     * Get list of debtors
     * @return stdClass debtors
     * {
     *    "data": [
     *        {
     *            "id": "deb_fT3xWQq3scF7cbIAv4knm0Oc",
     *            "names": [
     *                "Jack",
     *                "Napier"
     *            ],
     *            "address": {
     *                "addressLines": [
     *                    "1007 Mountain Drive",
     *                    "Wayne Manor"
     *                ],
     *                "city": "Gotham",
     *                "postCode": "1000",
     *                "countryCode": "US"
     *            },
     *            "vatNumber": "BE0326598547",
     *            "language": "fr",
     *            "currency": "EUR",
     *            "email": "abbott@koalapay.com,costello@koalapay.com",
     *            "reference": "CUS-102-12",
     *            "accountant": {
     *                "reference": "TECN",
     *                "number": "326587"
     *            },
     *            "contactPerson": "John Appleseed",
     *            "phoneNumbers": [
     *                "+32 487 29 43"
     *            ],
     *            "comments": "Debtor comments",
     *            "taxRate": "ESVAT21",
     *            "paymentTerm": "30",
     *            "url": "https://connect.koalapay.com/api/debtor/deb_1nhvjjghz9Nc84a3rwpP0zLh"
     *        }
     *    ],
     *    "meta": {
     *        "from": 51,
     *        "to": 100,
     *        "current_page": 2,
     *        "last_page": 3,
     *        "per_page": 50
     *    }
     * }
     */
    public function getDebtors()
    {
        $url = $this->url . 'debtors';
        $response = Requests::get($url, array(), array('hooks' => $this->hooks));
        return $response->data;
    }

    /**
     * Create new debtor
     * @param string $data debtor json data
     * Example
     * {
     *  "names": [
     *      "Jack",
     *      "Napier"
     *  ],
     *  "address": {
     *      "addressLines": [
     *          "1007 Mountain Drive",
     *          "Wayne Manor"
     *      ],
     *      "city": "Gotham",
     *      "postCode": "1000",
     *      "countryCode": "US"
     *  },
     *  "vatNumber": "BE0326598547",
     *  "language": "fr",
     *  "currency": "EUR",
     *  "email": "abbott@koalapay.com,costello@koalapay.com",
     *  "reference": "CUS-102-12",
     *  "accountant": {
     *      "reference": "TECN",
     *      "number": "326587"
     *  },
     *  "contactPerson": "John Appleseed",
     *  "phoneNumbers": [
     *      "+32 487 29 43"
     *  ],
     *  "comments": "Debtor comments",
     *  "taxRate": "ESVAT21",
     *  "paymentTerm": "30"
     * }
     * @return stdClass debtor
     * Example
     * {
     *  "id": "deb_fT3xWQq3scF7cbIAv4knm0Oc",
     *  "names": [
     *      "Jack",
     *      "Napier"
     *  ],
     *  "address": {
     *      "addressLines": [
     *          "1007 Mountain Drive",
     *          "Wayne Manor"
     *      ],
     *      "city": "Gotham",
     *      "postCode": "1000",
     *      "countryCode": "US"
     *  },
     *  "vatNumber": "BE0326598547",
     *  "language": "fr",
     *  "currency": "EUR",
     *  "email": "abbott@koalapay.com,costello@koalapay.com",
     *  "reference": "CUS-102-12",
     *  "accountant": {
     *      "reference": "TECN",
     *      "number": "326587"
     *  },
     *  "contactPerson": "John Appleseed",
     *  "phoneNumbers": [
     *      "+32 487 29 43"
     *  ],
     *  "comments": "Debtor comments",
     *  "taxRate": "ESVAT21",
     *  "paymentTerm": "30"
     * }
     * ... OR ...
     * {
     *  "message": "Could not create debtor",
     *  "errors": {
     *      "vatNumber": [
     *          {
     *              "code": "vat_number_rule_error",
     *              "message": "Invalid VAT number (e.g : BE0123456789 )"
     *          }
     *      ]
     *  }
     * }
     */
    public function addDebtor($data)
    {
        $url = $this->url . 'debtors';
        $response = Requests::post($url, array(), $data, array('hooks' => $this->hooks));
        return $response->data;
    }

    /**
     * Get debtor
     * @param string $id_debtor
     * @return stdClass debtor
     * {
     *    "id": "deb_fT3xWQq3scF7cbIAv4knm0Oc",
     *    "names": [
     *        "Jack",
     *        "Napier"
     *    ],
     *    "address": {
     *        "addressLines": [
     *        "1007 Mountain Drive",
     *        "Wayne Manor"
     *        ],
     *        "city": "Gotham",
     *        "postCode": "1000",
     *        "countryCode": "US"
     *    },
     *    "vatNumber": "BE0326598547",
     *    "language": "fr",
     *    "currency": "EUR",
     *    "email": "abbott@koalapay.com,costello@koalapay.com",
     *    "reference": "CUS-102-12",
     *    "accountant": {
     *        "reference": "TECN",
     *        "number": "326587"
     *    },
     *    "contactPerson": "John Appleseed",
     *    "phoneNumbers": [
     *        "+32 487 29 43"
     *    ],
     *    "comments": "Debtor comments",
     *    "taxRate": "ESVAT21",
     *    "paymentTerm": "30",
     *    "url": "https://connect.koalapay.com/api/debtor/deb_1nhvjjghz9Nc84a3rwpP0zLh"
     * }
     * ... OR ...
     * {
     *    "message": "Debtor not found"
     * }
     */
    public function getDebtor($id_debtor)
    {
        $url = $this->url . "debtors/$id_debtor";
        $response = Requests::get($url, array(), array('hooks' => $this->hooks));
        return $response->data;
    }

    /**
     * Update debtor
     * @param string $id_debtor
     * @param string $data debtor json data
     * Example
     * {
     *  "names": [
     *      "Jack",
     *      "Napier"
     *  ],
     *  "address": {
     *      "addressLines": [
     *          "1007 Mountain Drive",
     *          "Wayne Manor"
     *      ],
     *      "city": "Gotham",
     *      "postCode": "1000",
     *      "countryCode": "US"
     *  },
     *  "vatNumber": "BE0326598547",
     *  "language": "fr",
     *  "currency": "EUR",
     *  "email": "abbott@koalapay.com,costello@koalapay.com",
     *  "reference": "CUS-102-12",
     *  "accountant": {
     *      "reference": "TECN",
     *      "number": "326587"
     *  },
     *  "contactPerson": "John Appleseed",
     *  "phoneNumbers": [
     *      "+32 487 29 43"
     *  ],
     *  "comments": "Debtor comments",
     *  "taxRate": "ESVAT21",
     *  "paymentTerm": "30"
     * }
     * @return stdClass debtor
     * {
     *    "id": "deb_fT3xWQq3scF7cbIAv4knm0Oc",
     *    "names": [
     *        "Jack",
     *        "Napier"
     *    ],
     *    "address": {
     *        "addressLines": [
     *            "1007 Mountain Drive",
     *            "Wayne Manor"
     *        ],
     *        "city": "Gotham",
     *        "postCode": "1000",
     *        "countryCode": "US"
     *    },
     *    "vatNumber": "BE0326598547",
     *    "language": "fr",
     *    "currency": "EUR",
     *    "email": "abbott@koalapay.com,costello@koalapay.com",
     *    "reference": "CUS-102-12",
     *    "accountant": {
     *        "reference": "TECN",
     *        "number": "326587"
     *    },
     *    "contactPerson": "John Appleseed",
     *    "phoneNumbers": [
     *        "+32 487 29 43"
     *    ],
     *    "comments": "Debtor comments",
     *    "taxRate": "ESVAT21",
     *    "paymentTerm": "30",
     *    "url": "https://connect.koalapay.com/api/debtor/deb_1nhvjjghz9Nc84a3rwpP0zLh"
     * }
     * ... OR ...
     * {
     *    "message": "Could not create debtor",
     *    "errors": {
     *        "vatNumber": [
     *            {
     *                "code": "vat_number_rule_error",
     *                "message": "Invalid VAT number (e.g : BE0123456789 )"
     *            }
     *        ]
     *    }
     * }
     */
    public function updateDebtor($id_debtor, $data)
    {
        $url = $this->url . "debtors/$id_debtor";
        $response = Requests::put($url, array(), $data, array('hooks' => $this->hooks));
        return $response->data;
    }

    /**
     * Delete debtor
     * @param string $id_debtor
     * @return stdClass response
     * {
     *    "message": "Debtor not found"
     * }
     */
    public function deleteDebtor($id_debtor)
    {
        $url = $this->url . "debtors/$id_debtor";
        $response = Requests::delete($url, array(), array('hooks' => $this->hooks));
        return $response->data;
    }

    /**
     * Get invoice list
     * @param string|null $id_debtor Debtor ID to get all the invoices associated with a debtor
     * OR null to get all the invoices of the current customer
     * @return stdClass invoices
     * {
     *    "data": [
     *        {
     *            "id": "inv_1nhvjHhKz9NcVTW3rwpYczLh",
     *            "debtor": {
     *                "id": "deb_1nhvjHhKz9NcVTW3rwpYczLh",
     *                "name": "John Doe",
     *                "url": "https://connect.koalapay.com/api/debtor/deb_1nhvjHhKz9NcVTW3rwpYczLh"
     *            },
     *            "status": "draft",
     *            "financingStatus": "financed",
     *            "total": 0,
     *            "url": "https://connect.koalapay.com/api/invoice/inv_1nhvjHhKz9NcVTW3rwpYczLh"
     *        }
     *    ],
     *    "meta": {
     *        "from": 51,
     *        "to": 100,
     *        "current_page": 2,
     *        "last_page": 3,
     *        "per_page": 50
     *    }
     * }
     */
    public function getInvoices($id_debtor = null)
    {
        if ($id_debtor) {
            $url = $this->url . "debtors/$id_debtor/invoices";
        } else {
            $url = $this->url . 'invoices';
        }
        $response = Requests::get($url, array(), array('hooks' => $this->hooks));
        return $response->data;
    }

    /**
     * Get invoice infos
     * @param string $id_invoice invoice ID
     * @return stdClass invoice or message
     * {
     *    "id": "inv_1nhvjHhKz9NcVTW3rwpYczLh",
     *    "debtor": {
     *        "id": "deb_1nhvjHhKz9NcVTW3rwpYczLh",
     *        "name": "John Doe",
     *        "url": "https://connect.koalapay.com/api/debtor/deb_1nhvjHhKz9NcVTW3rwpYczLh"
     *    },
     *    "status": "draft",
     *    "number": "2019/001",
     *    "dueDate": "1985-12-12",
     *    "date": "1985-12-12",
     *    "discountRate": 0.1,
     *    "downPayment": 99.99,
     *    "currency": "EUR",
     *    "financingRequested": true,
     *    "financingStatus": "financed",
     *    "delivery": {
     *        "type": "DELIVERY",
     *        "date": "1985-12-12",
     *        "address": {
     *            "addressLines": [
     *                "1007 Mountain Drive",
     *                "Wayne Manor"
     *            ],
     *            "city": "Gotham",
     *            "postCode": "1000",
     *            "countryCode": "US"
     *        }
     *    },
     *    "payment": {
     *        "term": "30INV",
     *        "communication": {
     *            "type": "BE",
     *            "value": "+++012/3456/78939+++"
     *        },
     *        "instructions": "Please pay on the account above within 30 days after the invoice date"
     *    },
     *    "freeNotes": "Thank you for you confidence",
     *    "language": "fr",
     *    "balance": 2510.5,
     *    "lines": [
     *        {
     *            "id": "inl_g0em49q9N62RBSy5MfEcj9GQ",
     *            "product": {
     *            "id": "pro_mrkIvO3aKz0MVZ6Gf86XCp4x",
     *            "reference": "TABLE/BL",
     *            "description": {
     *            "fr": "Table en bois noir",
     *            "nl": "Zwarte houten tafel",
     *            "de": "Schwarzer Holztisch",
     *            "en": "Black wooden table",
     *            "es": "Mesa de madera negra",
     *            "ca": "Taula de fusta negra"
     *            },
     *            "unitPrice": "270.50",
     *            "unit": "kg",
     *            "accountingCode": "8994/TB",
     *            "taxRate": "ESVAT21",
     *            "category": {
     *            "id": "cat_muiIvO3aKz078Z6Gf86XCp4x",
     *            "description": "Cooking utils",
     *            "accountingCode": "90/XTB",
     *            "url": "https://connect.koalapay.com/api/product-category/cat_1nhvjjghz9Nc84a3rwpP0zLh"
     *            },
     *            "url": "https://connect.koalapay.com/api/product/pro_1nL5jH4hz9Nc84a3rwpP0zLh"
     *            },
     *            "description": "Stabilised sand",
     *            "unitPrice": 9.25,
     *            "unit": "m²",
     *            "quantity": 253,
     *            "taxRate": "BEVAT21",
     *            "discountRate": 0.1,
     *            "displayOrder": 1
     *        }
     *    ],
     *    "totals": {
     *        "net": "75.00",
     *        "tax": "15.75",
     *        "total": "90.75",
     *        "taxes": [
     *            {
     *                "rate": "0.2100",
     *                "net": "75.00",
     *                "tax": "15.75",
     *                "label": "21%",
     *                "extra": "[]",
     *                "legalNotice": ""
     *           }
     *        ],
     *        "extra": "[]",
     *        "reductions": "[]",
     *        "availableExtraTaxes": "[]",
     *        "vatExempt": "",
     *        "vatExemptLegal": "",
     *        "discount": ""
     *    },
     *    "fields": [
     *       {
     *            "key": "Tracking ID",
     *            "value": "RRH-362140-0012"
     *        }
     *    ]
     * }
     * ... OR ...
     * {
     *    "message": "Invoice not found"
     * }
     */
    public function getInvoice($id_invoice)
    {
        $url = $this->url . "invoices/$id_invoice";
        $response = Requests::get($url, array(), array('hooks' => $this->hooks));
        return $response->data;
    }

    /**
     * Get pdf invoice content
     * @param string $id_invoice invoice ID
     * @return mixed
     */
    public function getPdfInvoice($id_invoice)
    {
        $url = $this->url . "invoices/$id_invoice/pdf";
        $response = Requests::get($url, array(), array('hooks' => $this->hooks));
        return $response;
    }

    /**
     * Create new invoice
     * @param string $data invoice json data
     * Example (all the fields are optional)
     * {
     *  "debtorId": "deb_Fu1uLSXFLD0I7W37x0dUAKUB",
     *  "number": "123",
     *  "date": "1985-12-12",
     *  "dueDate": "1985-12-12",
     *  "discountRate": 0.1,
     *  "downPayment": 99.99,
     *  "currency": "EUR",
     *  "delivery": {
     *      "type": "DELIVERY",
     *      "date": "1985-12-12",
     *      "address": {
     *          "addressLines": [
     *              "1007 Mountain Drive",
     *              "Wayne Manor"
     *          ],
     *          "city": "Gotham",
     *          "postCode": "1000",
     *          "countryCode": "US"
     *      }
     *  },
     *  "payment": {
     *      "term": "30INV",
     *      "communication": {
     *          "type": "BE",
     *          "value": "+++012/3456/78939+++"
     *      },
     *      "instructions": "Please pay on the account above within 30 days after the invoice date"
     *  },
     *  freeNotes": "Thank you for you confidence"
     * }
     * @return stdClass invoice
     * {
     *    "data": [
     *        {
     *            "id": "inv_1nhvjHhKz9NcVTW3rwpYczLh",
     *            "debtor": {
     *                "id": "deb_1nhvjHhKz9NcVTW3rwpYczLh",
     *                "name": "John Doe",
     *                "url": "https://connect.koalapay.com/api/debtor/deb_1nhvjHhKz9NcVTW3rwpYczLh"
     *            },
     *            "status": "draft",
     *            "financingStatus": "financed",
     *            "total": 0,
     *            "url": "https://connect.koalapay.com/api/invoice/inv_1nhvjHhKz9NcVTW3rwpYczLh"
     *        }
     *    ],
     *    "meta": {
     *        "from": 51,
     *        "to": 100,
     *        "current_page": 2,
     *        "last_page": 3,
     *        "per_page": 50
     *    }
     * }
     */
    public function addInvoice($data)
    {
        $url = $this->url . 'invoices';
        $response = Requests::post($url, array(), $data, array('hooks' => $this->hooks));
        return $response->data;
    }

    /**
     * Update invoice
     * @param string $id_invoice
     * @param string $data invoice json data
     * Example (all the fields are optional)
     * {
     *  "debtorId": "deb_Fu1uLSXFLD0I7W37x0dUAKUB",
     *  "number": "123",
     *  "date": "1985-12-12",
     *  "dueDate": "1985-12-12",
     *  "discountRate": 0.1,
     *  "downPayment": 99.99,
     *  "currency": "EUR",
     *  "delivery": {
     *      "type": "DELIVERY",
     *      "date": "1985-12-12",
     *      "address": {
     *          "addressLines": [
     *              "1007 Mountain Drive",
     *              "Wayne Manor"
     *          ],
     *          "city": "Gotham",
     *          "postCode": "1000",
     *          "countryCode": "US"
     *      }
     *  },
     *  "payment": {
     *      "term": "30INV",
     *      "communication": {
     *          "type": "BE",
     *          "value": "+++012/3456/78939+++"
     *      },
     *      "instructions": "Please pay on the account above within 30 days after the invoice date"
     *  },
     *  freeNotes": "Thank you for you confidence"
     * }
     * @return stdClass invoice infos
     * {
     *    "data": [
     *        {
     *            "id": "inv_1nhvjHhKz9NcVTW3rwpYczLh",
     *            "debtor": {
     *                "id": "deb_1nhvjHhKz9NcVTW3rwpYczLh",
     *                "name": "John Doe",
     *                "url": "https://connect.koalapay.com/api/debtor/deb_1nhvjHhKz9NcVTW3rwpYczLh"
     *            },
     *            "status": "draft",
     *            "financingStatus": "financed",
     *            "total": 0,
     *            "url": "https://connect.koalapay.com/api/invoice/inv_1nhvjHhKz9NcVTW3rwpYczLh"
     *        }
     *    ],
     *    "meta": {
     *        "from": 51,
     *        "to": 100,
     *        "current_page": 2,
     *        "last_page": 3,
     *        "per_page": 50
     *    }
     * }
     */
    public function updateInvoice($id_invoice, $data)
    {
        $url = $this->url . "invoices/$id_invoice";
        $response = Requests::put($url, array(), $data, array('hooks' => $this->hooks));
        return $response->data;
    }

    /**
     * Delete invoice
     * @param string $id_invoice
     * @return stdClass response
     * {
     *    "message": "The invoice was successfully deleted"
     * }
     */
    public function deleteInvoice($id_invoice)
    {
        $url = $this->url . "invoices/$id_invoice";
        $response = Requests::delete($url, array(), array('hooks' => $this->hooks));
        return $response;
    }

    /**
     * Send invoice
     * @param string $id_invoice
     * @param string $data json data
     * Example (all the fields are optional)
     * {
     *  "methods": [
     *      "eInvoice",
     *      "email",
     *      "post"
     *  ]
     *  "email": {
     *      "from": "string",
     *      "to": "string",
     *      "cc": "string",
     *      "subject": "string",
     *      "message": "string",
     *      "copy": true,
     *      "receipt": true
     *  },
     *  "eInvoice": {
     *      "attachments": true
     *  },
     *  "post": {
     *      "attachments": true,
     *      "envelope": "C4",
     *      "type": "EXPRESS"
     *  }
     * }
     * @return stdClass debtor infos
     * {
     *    "data": [
     *        {
     *            "id": "inv_1nhvjHhKz9NcVTW3rwpYczLh",
     *            "debtor": {
     *                "id": "deb_1nhvjHhKz9NcVTW3rwpYczLh",
     *                "name": "John Doe",
     *                "url": "https://connect.koalapay.com/api/debtor/deb_1nhvjHhKz9NcVTW3rwpYczLh"
     *            },
     *            "status": "draft",
     *            "financingStatus": "financed",
     *            "total": 0,
     *            "url": "https://connect.koalapay.com/api/invoice/inv_1nhvjHhKz9NcVTW3rwpYczLh"
     *        }
     *    ],
     *    "meta": {
     *        "from": 51,
     *        "to": 100,
     *        "current_page": 2,
     *        "last_page": 3,
     *        "per_page": 50
     *    }
     * }
     */
    public function sendInvoice($id_invoice, $data)
    {
        $url = $this->url . "invoices/$id_invoice/send";
        $response = Requests::post($url, array(), $data, array('hooks' => $this->hooks));
        return $response->data;
    }

    /**
     * Get Invoice Lines
     * @param string $id_invoice
     * @param string $data
     * @return stdClass invoice lines
     * Example
     * {
     *  "data": [
     *      {
     *          "id": "inl_g0em49q9N62RBSy5MfEcj9GQ",
     *          "product": {
     *              "id": "pro_mrkIvO3aKz0MVZ6Gf86XCp4x",
     *              "reference": "TABLE/BL",
     *              "description": {
     *                "fr": "Table en bois noir",
     *                  "nl": "Zwarte houten tafel",
     *                  "de": "Schwarzer Holztisch",
     *                  "en": "Black wooden table",
     *                  "es": "Mesa de madera negra",
     *                  "ca": "Taula de fusta negra"
     *              },
     *              "unitPrice": "270.50",
     *              "unit": "kg",
     *              "accountingCode": "8994/TB",
     *              "taxRate": "ESVAT21",
     *              "category": {
     *                  "id": "cat_muiIvO3aKz078Z6Gf86XCp4x",
     *                  "description": "Cooking utils",
     *                  "accountingCode": "90/XTB",
     *                  "url": "https://connect.koalapay.com/api/product-category/cat_1nhvjjghz9Nc84a3rwpP0zLh"
     *              },
     *              "url": "https://connect.koalapay.com/api/product/pro_1nL5jH4hz9Nc84a3rwpP0zLh"
     *          },
     *          "description": "Stabilised sand",
     *          "unitPrice": 9.25,
     *          "unit": "m²",
     *          "quantity": 253,
     *          "taxRate": "BEVAT21",
     *          "discountRate": 0.1,
     *          "displayOrder": 1
     *      }
     *  ]
     * }
     */
    public function getInvoiceLines($id_invoice, $data)
    {
        $url = $this->url . "invoices/$id_invoice/lines";
        $response = Requests::post($url, array(), $data, array('hooks' => $this->hooks));
        return $response->data;
    }

    /**
     * Update Invoice Lines
     * @param string $id_invoice
     * @param string $data json data
     * Example
     * {
     *  "lines": [
     *      {
     *          "id": "lin_CLYLfLhgm6OzJKypAppXihN5",
     *          "action": "update",
     *          "description": "Stabilised sand",
     *          "unitPrice": 9.25,
     *          "unit": "m²",
     *          "quantity": 253,
     *          "taxRate": "BEVAT21",
     *          "discountRate": 0.1,
     *          "displayOrder": 1
     *      }
     *  ]
     * }
     * @return stdClass invoice lines infos
     * Example
     * {
     *  "data": [
     *      {
     *          "id": "inl_g0em49q9N62RBSy5MfEcj9GQ",
     *          "product": {
     *              "id": "pro_mrkIvO3aKz0MVZ6Gf86XCp4x",
     *              "reference": "TABLE/BL",
     *              "description": {
     *                "fr": "Table en bois noir",
     *                  "nl": "Zwarte houten tafel",
     *                  "de": "Schwarzer Holztisch",
     *                  "en": "Black wooden table",
     *                  "es": "Mesa de madera negra",
     *                  "ca": "Taula de fusta negra"
     *              },
     *              "unitPrice": "270.50",
     *              "unit": "kg",
     *              "accountingCode": "8994/TB",
     *              "taxRate": "ESVAT21",
     *              "category": {
     *                  "id": "cat_muiIvO3aKz078Z6Gf86XCp4x",
     *                  "description": "Cooking utils",
     *                  "accountingCode": "90/XTB",
     *                  "url": "https://connect.koalapay.com/api/product-category/cat_1nhvjjghz9Nc84a3rwpP0zLh"
     *              },
     *              "url": "https://connect.koalapay.com/api/product/pro_1nL5jH4hz9Nc84a3rwpP0zLh"
     *          },
     *          "description": "Stabilised sand",
     *          "unitPrice": 9.25,
     *          "unit": "m²",
     *          "quantity": 253,
     *          "taxRate": "BEVAT21",
     *          "discountRate": 0.1,
     *          "displayOrder": 1
     *      },
     *      "meta": {
     *          "created": 1,
     *          "updated": 2,
     *          "deleted": 3
     *      }
     *  ]
     * }
     */
    public function updateInvoiceLines($id_invoice, $data)
    {
        $url = $this->url . "invoices/$id_invoice/lines/bulk";
        $response = Requests::post($url, array(), $data, array('hooks' => $this->hooks));
        return $response->data;
    }

    /**
     * Check if an invoice is financeable
     * @param string $id_invoice
     * @param string $data
     * Example
     * {
     *    "requestIfEligible": false
     * }
     * @return stdClass response
     * {
     *    "data": {
     *        "eligible": true,
     *        "requested": false,
     *        "reason": "You don't have sufficient funds to finance this invoice"
     *    }
     * }
     */
    public function checkEligibility($id_invoice, $data)
    {
        $url = $this->url . "invoices/$id_invoice/financing/check-eligibility";
        $response = Requests::post($url, array(), $data, array('hooks' => $this->hooks));
        return $response->data;
    }
}
