<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2021 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

/**
 * Class KoalapayCartInvoice
 */
class KoalapayCartInvoice extends ObjectModel
{
    /** @var int $id Cart Invoice ID */
    public $id;

    /** @var int $id_cart Customer Cart ID */
    public $id_cart;

    /** @var string $id_invoice Invoice ID */
    public $id_invoice;

    /** @var string $invoice_number Invoice number */
    public $invoice_number;

    /** @var string $status invoice status */
    public $status;

    /** @var string $financing_status financing status */
    public $financing_status;

    /** @var string $total of invoice */
    public $total;

    /** @var bool is_sent */
    public $is_sent;

    /**
     * @see ObjectModel::$definition
     */
    public static $definition = array(
        'table' => 'koalapay_cart_invoice',
        'primary' => 'id_cart_invoice',
        'fields' => array(
            'id_cart' => array('type' => self::TYPE_INT, 'validate' => 'isUnsignedInt', 'required' => true),
            'id_invoice' => array(
                'type' => self::TYPE_STRING,
                'validate' => 'isString',
                'required' => true,
                'size' => 255
            ),
            'invoice_number' => array(
                'type' => self::TYPE_STRING,
                'validate' => 'isString',
                'size' => 255
            ),
            'status' => array(
                'type' => self::TYPE_STRING,
                'validate' => 'isString',
                'size' => 255
            ),
            'financing_status' => array(
                'type' => self::TYPE_STRING,
                'validate' => 'isString',
                'size' => 255
            ),
            'total' => array('type' => self::TYPE_FLOAT, 'validate' => 'isPrice', 'required' => true),
            'is_sent' => array('type' => self::TYPE_BOOL, 'validate' => 'isBool', 'required' => true),
        ),
    );

    /**
     * Get Invoice by Cart
     * @param mixed $cart Cart or ID
     * @return string cart ID
     */
    public static function getIdInvoice($cart)
    {
        if (Validate::isLoadedObject($cart)) {
            $id_cart = $cart->id;
        } else {
            $id_cart = $cart;
        }
        $sql = 'SELECT `id_invoice`
			FROM `' . _DB_PREFIX_ . 'koalapay_cart_invoice`
			WHERE `id_cart` = ' . (int)$id_cart;

        return Db::getInstance()->getValue($sql);
    }

    /**
     * Get Cart by Invoice ID
     * @param string $id_invoice Invoice ID
     * @return int customer ID
     */
    public static function getIdCart($id_invoice)
    {
        $sql = 'SELECT `id_cart`
			FROM `' . _DB_PREFIX_ . 'koalapay_cart_invoice`
			WHERE `id_invoice` = ' . (int)$id_invoice;

        return Db::getInstance()->getValue($sql);
    }

    /**
     * Get cart invoice by Order
     * @param Cart|Order $object Cart or Order
     * @return KoalapayCartInvoice
     */
    public static function getCartInvoice($object)
    {
        if (Validate::isLoadedObject($object)) {
            if ($object instanceof Cart) {
                $cart = $object;
            } elseif ($object instanceof Order) {
                $cart = new Cart($object->id_cart);
            }
            if (Validate::isLoadedObject($cart)) {
                $sql = 'SELECT MAX(`id_cart_invoice`)
                    FROM `' . _DB_PREFIX_ . 'koalapay_cart_invoice`
                    WHERE `id_cart` = ' . (int)$cart->id;

                return new KoalapayCartInvoice(Db::getInstance()->getValue($sql));
            }
        }

        return null;
    }

    /**
     * Delete Cart Invoice
     * @param mixed $cart Cart or ID cart
     * @return bool
     */
    public static function deleteByCart($cart)
    {
        if (Validate::isLoadedObject($cart)) {
            $id_cart = $cart->id;
        } else {
            $id_cart = $cart;
        }

        return Db::getInstance()->delete('koalapay_cart_invoice', 'id_cart = ' . (int)$id_cart);
    }

    /**
     * Marks invoice as sent
     * @param string $id_invoice Invoice ID
     * @param string $invoice_number Invoice #
     * @param string|null $status status of invoice
     * @param string|null $financing_status financing status of invoice
     * @param float $total total of invoice
     * @throws PrestaShopException
     */
    public static function invoiceSent(
        $id_invoice,
        $invoice_number,
        $status,
        $financing_status,
        $total
    ) {
        $sql = 'SELECT `id_cart_invoice`
                    FROM `' . _DB_PREFIX_ . 'koalapay_cart_invoice`
                    WHERE id_invoice = "' . pSQL($id_invoice) . '"';
        $rows = Db::getInstance()->executeS($sql);
        foreach ($rows as $row) {
            $cart_invoice = new KoalapayCartInvoice($row['id_cart_invoice']);
            if (Validate::isLoadedObject($cart_invoice)) {
                $cart_invoice->invoice_number = $invoice_number;
                if ($status) {
                    $cart_invoice->status = $status;
                }
                if ($financing_status) {
                    $cart_invoice->financing_status = $financing_status;
                }
                $cart_invoice->total = $total;
                $cart_invoice->is_sent = true;
                $cart_invoice->save();
            }
        }
    }

    /**
     * Refresh invoice status/total
     * @param string $id_invoice Invoice ID
     * @param string|null $status status of invoice
     * @param string|null $financing_status financing status of invoice
     * @param float $total total of invoice
     * @return bool
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public static function setInvoiceStatus($id_invoice, $status, $financing_status, $total)
    {
        $sql = 'SELECT `id_cart_invoice`
                    FROM `' . _DB_PREFIX_ . 'koalapay_cart_invoice`
                    WHERE id_invoice = "' . pSQL($id_invoice) . '"';
        $rows = Db::getInstance()->executeS($sql);
        foreach ($rows as $row) {
            $cart_invoice = new KoalapayCartInvoice($row['id_cart_invoice']);
            if (Validate::isLoadedObject($cart_invoice)) {
                if ($status) {
                    $cart_invoice->status = $status;
                }
                if ($financing_status) {
                    $cart_invoice->financing_status = $financing_status;
                }
                $cart_invoice->total = $total;
                $cart_invoice->save();
            }
        }
    }

    /**
     * Get abandoned invoices (cart was deleted/abandoned or order has other payment)
     * @param int $days_back number of days that a cart is considered abandoned
     * @return array invoices
     * @throws PrestaShopDatabaseException
     */
    public static function getAbandonedInvoices($days_back)
    {
        $sec = (int)$days_back * 24 * 60 * 60;
        $date = pSQL(date('Y-m-d H:i:00', time()));
        $sql = "SELECT  kci.`id_cart_invoice`, kci.`id_cart`, kci.`id_invoice` 
			FROM `" . _DB_PREFIX_ . "koalapay_cart_invoice` kci
			LEFT JOIN `" . _DB_PREFIX_ . "cart` c ON c.id_cart = kci.id_cart
			LEFT JOIN `" . _DB_PREFIX_ . "orders` o ON o.id_cart = kci.id_cart
			WHERE c.`id_cart` IS NULL 
			    OR (o.`id_order` IS NULL AND TIME_TO_SEC(TIMEDIFF('" . $date . "', c.`date_add`)) > $sec)
			    OR (o.`id_order` > 0 AND o.module <> 'koalapay')";

        return Db::getInstance()->executeS($sql);
    }

    /**
     * Get invoices to refresh
     * @param int $days_back number of days that a invoice can be refreshed
     * @return array invoices
     * @throws PrestaShopDatabaseException
     */
    public static function getInvoicesToRefresh($days_back)
    {
        $sec = (int)$days_back * 24 * 60 * 60;
        $sql = "SELECT  kci.`id_cart_invoice`, kci.`id_cart`, kci.`id_invoice` 
			FROM `" . _DB_PREFIX_ . "koalapay_cart_invoice` kci
			INNER JOIN `" . _DB_PREFIX_ . "cart` c ON c.id_cart = kci.id_cart
			INNER JOIN `" . _DB_PREFIX_ . "orders` o ON o.id_cart = kci.id_cart
			WHERE (TIME_TO_SEC(TIMEDIFF('" . pSQL(date('Y-m-d H:i:00', time())) . "', c.`date_add`)) < $sec)
			    AND (o.`id_order` > 0 AND o.module = 'koalapay')";

        return Db::getInstance()->executeS($sql);
    }
}
