<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2021 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

/**
 * Class KoalapayTaxRates
 */
class KoalapayTaxRates
{
    /**
     * @var array()
     */
    private static $tax_rates;

    public static function getTaxRates()
    {
        if (!self::$tax_rates) {
            $json_file = dirname(__FILE__) . '/../tax-rates.json';
            if (!file_exists($json_file) || !fopen($json_file, 'r')) {
                throw new Exception("Tax rates config: unable to open file $json_file!");
            }
            $tax_rates_json = Tools::file_get_contents($json_file);
            $result = json_decode($tax_rates_json, true);
            if (!$result) {
                throw new Exception("Tax rates config: invalid json format - ".json_last_error_msg());
            }
            self::$tax_rates = $result;
        }

        return self::$tax_rates;
    }

    /**
     * Get tax rate to add in invoice line
     * @param Cart $cart
     * @param $product
     * @return string tax rate
     * @throws Exception
     */
    public static function getTaxRateByCart(Cart $cart, $product)
    {
        if (!method_exists($cart, 'getProductAddressId')) {
            // backward compatibility
            if (Configuration::get('PS_TAX_ADDRESS_TYPE') == 'id_address_invoice') {
                $product_address_id = (int)$cart->id_address_invoice;
            } else {
                $product_address_id = (int)$product['id_address_delivery'];
            }
        } else {
            $product_address_id = $cart->getProductAddressId($product);
        }
        $address = new Address($product_address_id);
        return self::getTaxRate($product['rate'], $address->id_country);
    }

    /**
     * Get tax rate to add in invoice discount line
     * @param Cart $cart
     * @return string|null
     * @throws Exception
     */
    public static function getDiscountTaxRateByCart(Cart $cart)
    {
        $tax_rate = null;
        $rate = 0;
        foreach ($cart->getProducts() as $product) {
            if ($product['rate'] > $rate) {
                $tax_rate = self::getTaxRateByCart($cart, $product);
            }
        }
        return $tax_rate;
    }

    /**
     * Get tax rate to add in invoice line
     * @param float $rate
     * @param int|null $id_country ID country (PS_COUNTRY_DEFAULT if null)
     * @return string tax rate
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public static function getTaxRate($rate, $id_country = null)
    {
        $id_country_default = Configuration::get('PS_COUNTRY_DEFAULT');
        if (!$id_country) {
            $id_country = $id_country_default;
        }
        $country = new Country((int)$id_country);
        if (!Validate::isLoadedObject($country)) {
            throw new Exception("Tax rates config: country (ID $id_country) not found");
        }
        $countries = array($country->iso_code);
        if ($id_country != $id_country_default) {
            $country_default = new Country((int)$id_country_default);
            if (Validate::isLoadedObject($country_default)) {
                $countries[] = $country_default->iso_code;
            }
        }
        $tax_rates = self::getTaxRates();
        foreach ($countries as $iso_code) {
            if (isset($tax_rates[$iso_code])) {
                foreach ($tax_rates[$iso_code] as $code => $infos) {
                    if ((float) $infos['value'] == (float) ($rate /  100)) {
                        return $code;
                    }
                }
            }
        }
        return null;
    }
}
