<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2021 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

require_once dirname(__FILE__) . '/KoalapayApi.php';

/**
 * Class Kpay
 */
class Kpay
{

    /** @var KoalapayApi $api */
    protected static $api;

    /** @var Koalapay $module */
    protected static $module;

    /**
     * Get Koalapay api instance
     * @return KoalapayApi
     */
    protected static function getApi()
    {
        if (!self::$api) {
            $api_url = Configuration::get('KOALAPAY_API_URL');
            $api_token = Configuration::get('KOALAPAY_API_TOKEN');
            self::$api = new KoalapayApi($api_url, $api_token);
        }
        return self::$api;
    }

    /**
     * Get module instance
     * @return Koalapay
     */
    protected static function getModule()
    {
        if (!self::$module) {
            self::$module = Module::getInstanceByName('koalapay');
        }
        return self::$module;
    }

    /**
     * Get translation for the given module text.
     *
     * Note: $specific parameter is mandatory for library files.
     * Otherwise, translation key will not match for Module library
     * when module is loaded with eval() Module::getModulesOnDisk()
     *
     * @param string $string String to translate
     * @param bool|string $specific filename to use in translation key
     * @param string|null $locale Locale to translate to
     *
     * @return string Translation
     */
    protected static function l($string, $specific = false, $locale = null)
    {
        return self::getModule()->l($string, $specific, $locale);
    }

    /**
     * Get language code by Cart
     * @param Cart $cart
     * @return string
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    protected static function getLanguage(Cart $cart)
    {
        $lang = new Language($cart->id_lang);
        return Tools::substr($lang->language_code, 0, 2);
    }

    /**
     * Throw Exception with correct error message
     * @param mixed $result
     * @param string $data body post sent
     * @return Exception
     * @throws Exception
     */
    protected static function throwException($result, $data = '')
    {
        $msg = self::l('Unknown Error');
        if (is_object($result)) {
            if (isset($result->message)) {
                $msg = $result->message;
            }
            if (isset($result->errors)) {
                $msg .= ': ' . json_encode($result->errors);
            }
            if ($data) {
                $msg .= ' | DATA SENT: ' . $data;
            }
        }
        throw new Exception($msg);
    }

    /**
     * Check if API is alive
     * @return bool|string true if alive or the error message
     * @see KoalapayApi::isAlive()
     */
    public static function apiAlive()
    {
        return self::getApi()->isAlive();
    }

    /**
     * Get Current customer info
     * @return stdClass current customer info
     * @see KoalapayApi::getCurrentCustomer()
     */
    public static function getCurrentCustomer()
    {
        $customer = self::getApi()->getCurrentCustomer();
        if (isset($customer->data)) {
            $customer = $customer->data;
        }
        return $customer;
    }

    /**
     * Get financing status of the current customer
     * @return stdClass financing status
     * @see KoalapayApi::getFinancingStatus()
     */
    public static function getFinancingStatus()
    {
        $status = self::getApi()->getFinancingStatus();
        if (isset($status->data)) {
            $status = $status->data;
        }
        return $status;
    }

    /**
     * Get list of debtors
     * @return stdClass debtors
     * @see KoalapayApi::getDebtors()
     */
    public static function getDebtors()
    {
        return self::getApi()->getDebtors();
    }

    /**
     * Get debtor by customer
     * @param Customer $customer
     * @return stdClass|null debtor or null if not exists
     * @see KoalapayApi::getDebtor()
     */
    public static function getDebtor(Customer $customer)
    {
        $id_debtor = KoalapayCustomerDebtor::getIdDebtor($customer);
        if (!$id_debtor) {
            return null;
        }
        $debtor = self::getApi()->getDebtor($id_debtor);
        if (isset($debtor->data)) {
            $debtor = $debtor->data;
        }
        if (!isset($debtor->id) || !$debtor->id) {
            KoalapayCustomerDebtor::deleteByCustomer($customer);
            return null;
        }
        return $debtor;
    }

    /**
     * Create new debtor
     * @param Cart $cart
     * @return stdClass
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     * * @throws Exception
     * @see KoalapayApi::addDebtor()
     */
    public static function addDebtor(Cart $cart)
    {
        $customer = new Customer($cart->id_customer);
        if (!Validate::isLoadedObject($customer)) {
            throw new Exception(self::l('Unknown Customer'));
        }
        $data = '';
        $is_new = false;
        $id_debtor = KoalapayCustomerDebtor::getIdDebtor($customer);
        if ($id_debtor) {
            $debtor = self::getApi()->getDebtor($id_debtor);
        } else {
            $data = self::getDebtorJsonData($cart);
            $debtor = self::getApi()->addDebtor($data);
            $is_new = true;
        }
        if (isset($debtor->data)) {
            $debtor = $debtor->data;
        }
        if (isset($debtor->id) && $debtor->id) {
            if ($is_new) {
                $customer_debtor = new KoalapayCustomerDebtor();
                $customer_debtor->id_customer = $customer->id;
                $customer_debtor->id_debtor = $debtor->id;
                $customer_debtor->add();
            }
        } else {
            self::throwException($debtor, $data);
        }
        return $debtor;
    }

    /**
     * Update debtor
     * @param string $id_debtor
     * @param Cart $cart
     * @return stdClass debtor
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     * @throws Exception
     * @see KoalapayApi::updateDebtor()
     */
    public static function updateDebtor($id_debtor, Cart $cart)
    {
        $customer = new Customer($cart->id_customer);
        if (!Validate::isLoadedObject($customer)) {
            throw new Exception(self::l('Customer not found'));
        }
        if (!$id_debtor) {
            throw new Exception(self::l('Debtor ID not found'));
        }
        $data = self::getDebtorJsonData($cart);
        $debtor = self::getApi()->updateDebtor($id_debtor, $data);
        if (isset($debtor->data)) {
            $debtor = $debtor->data;
        }
        if (!isset($debtor->id) || $debtor->id != $id_debtor) {
            self::throwException($debtor, $data);
        }
        return $debtor;
    }

    /**
     * Get invoice from Cart
     * @param Cart $cart
     * @return stdClass|null invoice or null if not exists
     * @see KoalapayApi::getInvoice()
     */
    public static function getInvoice(Cart $cart)
    {
        $id_invoice = KoalapayCartInvoice::getIdInvoice($cart);
        if (!$id_invoice) {
            return null;
        }
        $invoice = self::getApi()->getInvoice($id_invoice);
        if (isset($invoice->data)) {
            $invoice = $invoice->data;
        }
        if (!isset($invoice->id) || !$invoice->id) {
            KoalapayCartInvoice::deleteByCart($cart);
            return null;
        }
        return $invoice;
    }

    /**
     * Get pdf invoice content
     * @param string $id_invoice invoice ID
     * @return string pdf content
     * @throws Exception
     * @see KoalapayApi::getPdfInvoice()
     */
    public static function getPdfInvoice($id_invoice)
    {
        $response = self::getApi()->getPdfInvoice($id_invoice);
        if (isset($response->status_code)) {
            if ($response->status_code == 200) {
                return $response->body;
            }
            if ($response->status_code == 404) {
                throw new Exception(self::l('Invoice not found'));
            }
        }
        throw new Exception(self::l('Unable to retrieve invoice'));
    }

    /**
     * Get invoice list
     * @param Customer $customer
     * @return array invoices
     * @throws Exception
     * @see KoalapayApi::getInvoices()
     */
    public static function getInvoices(Customer $customer)
    {
        if (!Validate::isLoadedObject($customer)) {
            throw new Exception(self::l('Customer not found'));
        }
        $id_debtor = KoalapayCustomerDebtor::getIdDebtor($customer);
        if (!$id_debtor) {
            return array();
        }
        return self::getApi()->getInvoices($id_debtor);
    }

    /**
     * Create new invoice
     * @param Cart $cart
     * @param string|null $id_debtor
     * @return stdClass invoice
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     * @throws \PrestaShop\PrestaShop\Core\Localization\Exception\LocalizationException
     * @see KoalapayApi::addInvoice()
     */
    public static function addInvoice(Cart $cart, $id_debtor = null)
    {
        $customer = new Customer($cart->id_customer);
        if (!Validate::isLoadedObject($customer)) {
            throw new Exception(self::l('Customer not found'));
        }
        $data = '';
        $is_new = false;
        $id_invoice = KoalapayCartInvoice::getIdInvoice($cart);
        if ($id_invoice) {
            $invoice = self::getApi()->getInvoice($id_invoice);
        } else {
            if (!$id_debtor) {
                $id_debtor = KoalapayCustomerDebtor::getIdDebtor($customer);
            }
            if (!$id_debtor) {
                throw new Exception(self::l('Debtor ID not found'));
            }
            $data = self::getInvoiceJsonData($id_debtor, $cart);
            $invoice = self::getApi()->addInvoice($data);
            if (isset($invoice->data->totals->total)) {
                $cart_total = round($cart->getOrderTotal(), 2);
                $invoice_total = round($invoice->data->totals->total, 2);
                if ($cart_total != $invoice_total) {
                    // rounding issue?
                    if (self::deleteInvoice($invoice->data->id)) {
                        $data = self::getInvoiceJsonData($id_debtor, $cart, round($cart_total - $invoice_total, 2));
                        $invoice = self::getApi()->addInvoice($data);
                    }
                }
            }
            if (isset($invoice->data)) {
                $is_new = true;
                $invoice = $invoice->data;
            }
        }
        if (isset($invoice->id) && $invoice->id) {
            if ($is_new) {
                $cart_invoice = new KoalapayCartInvoice();
                $cart_invoice->id_cart = $cart->id;
                $cart_invoice->id_invoice = $invoice->id;
                $cart_invoice->total = (float)$invoice->totals->total;
                $cart_invoice->is_sent = false;
                $cart_invoice->add();
            }
        } else {
            self::throwException($invoice, $data);
        }
        return $invoice;
    }

    /**
     * Delete invoice via the api
     * @param string $id_invoice invoice ID
     * @return true|null|false true if successfully deleted, null if not found or false otherwise
     * @see KoalapayApi::deleteInvoice()
     */
    public static function deleteInvoice($id_invoice)
    {
        $response = self::getApi()->deleteInvoice($id_invoice);
        if (isset($response->status_code)) {
            if ($response->status_code == 200) {
                return true;
            }
            if ($response->status_code == 404) {
                return null;
            }
        }
        return false;
    }

    /**
     * Delete invoice by cart
     * @param mixed $cart Cart or ID cart
     * @return bool true or false if invoice did not exist
     * @throws Exception
     */
    public static function deleteInvoiceByCart($cart)
    {
        $id_invoice = KoalapayCartInvoice::getIdInvoice($cart);
        if ($id_invoice) {
            if (Kpay::deleteInvoice($id_invoice) === false) {
                $err = sprintf(self::l('Invoice %s was not deleted via the api'), $id_invoice);
                throw new Exception($err);
            }
            KoalapayCartInvoice::deleteByCart($cart);
            return true;
        }
        return false;
    }

    /**
     * Check if an invoice is financeable
     * @param string $id_invoice
     * @return stdClass response
     * @throws Exception
     * @see KoalapayApi::checkEligibility()
     */
    public static function checkEligibility($id_invoice)
    {
        $data = '{
            "requestIfEligible": false
        }';
        $eligibility = self::getApi()->checkEligibility($id_invoice, $data);
        if (isset($eligibility->data)) {
            $eligibility = $eligibility->data;
        } else {
            self::throwException($eligibility, $data);
        }
        return $eligibility;
    }

    /**
     * Send Invoice from cart
     * @param Cart $cart
     * @return stdClass invoice updated
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public static function sendInvoice(Cart $cart)
    {
        $invoice = Kpay::getInvoice($cart);
        if (!isset($invoice->id)) {
            throw new Exception(sprintf(
                self::l('Invoice not found for Cart (ID %s)'),
                $cart->id
            ));
        }
        $data = self::getSendInvoiceJsonData($cart);
        $response = self::getApi()->sendInvoice($invoice->id, $data);
        if (isset($response->data->id) && $response->data->id == $invoice->id) {
            $updated_invoice = $response->data;
            $status = null;
            if (isset($updated_invoice->status)) {
                $status = $updated_invoice->status;
            }
            $financing_status = null;
            if (isset($updated_invoice->financingStatus)) {
                $financing_status = $updated_invoice->financingStatus;
            }
        } else {
            self::throwException($response, $data);
        }
        KoalapayCartInvoice::invoiceSent(
            $invoice->id,
            $updated_invoice->number,
            $status,
            $financing_status,
            (float)$updated_invoice->totals->total
        );
        return $updated_invoice;
    }

    /**
     * Refresh Status of invoice from cart
     * @param Cart $cart
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public static function refreshStatus(Cart $cart)
    {
        $invoice = Kpay::getInvoice($cart);
        if (!isset($invoice->id)) {
            throw new Exception(sprintf(
                self::l('Invoice not found for Cart (ID %s)'),
                $cart->id
            ));
        }
        $status = null;
        if (isset($invoice->status)) {
            $status = $invoice->status;
        }
        $financing_status = null;
        if (isset($invoice->financingStatus)) {
            $financing_status = $invoice->financingStatus;
        }
        KoalapayCartInvoice::setInvoiceStatus(
            $invoice->id,
            $status,
            $financing_status,
            (float)$invoice->totals->total
        );
    }



    /**
     * Get debtor in json format from Customer
     * @param Cart $cart
     * @return string debtor in json format
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    protected static function getDebtorJsonData(Cart $cart)
    {
        $customer = new Customer($cart->id_customer);
        $currency = new Currency($cart->id_currency);
        $address = new Address($cart->id_address_invoice, $cart->id_lang);
        $country = new Country($address->id_country);
        $address_lines = array($address->address1);
        if ($address->address2) {
            $address_lines[] = $address->address2;
        }
        $phones = array();
        if ($address->phone_mobile) {
            $phones[] = $address->phone_mobile;
        }
        if ($address->phone) {
            $phones[] = $address->phone;
        }
        $phone_numbers = '';
        if (count($phones)) {
            $phone_numbers = '"phoneNumbers": ' . json_encode($phones) . ',';
        }
        $vat_number = '';
        if ($address->vat_number) {
            $vat_number = '"vatNumber": ' . json_encode($address->vat_number) . ',';
        }
        return '{
            "names": [' . json_encode($address->firstname . ' ' . $address->lastname) . '],
            "address": {
                "addressLines": ' . json_encode($address_lines) . ',
                "city": ' . json_encode($address->city) . ',
                "postCode": ' . json_encode($address->postcode) . ',
                "countryCode": ' . json_encode($country->iso_code) . '
            },
            ' . $vat_number . '
            "language": ' . json_encode(self::getLanguage($cart)) . ',
            "currency": ' . json_encode($currency->iso_code) . ',
            "email": ' . json_encode($customer->email) . ',
            "reference": ' . json_encode("CUS-" . $customer->id) . ',
            "accountant": {
                "reference": ' . json_encode("PRESTA-CUS-" . $customer->id) . ',
                "number": ' . json_encode($customer->id) . '
            },
            "contactPerson": ' . json_encode($customer->firstname . ' ' . $customer->lastname) . ',
            ' . $phone_numbers . '
            "comments": ' . json_encode("Automatically created by Koalapay PrestaShop Module.") . '
        }';
    }

    /**
     * Get invoice in json format from cart
     * @param string $id_debtor
     * @param Cart $cart
     * @param float $rounding_correction
     * @return string invoice in json format
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     * @throws \PrestaShop\PrestaShop\Core\Localization\Exception\LocalizationException
     */
    protected static function getInvoiceJsonData($id_debtor, Cart $cart, $rounding_correction = 0)
    {
        $currency = new Currency($cart->id_currency);
        $payment_term = 30;
        return '{
            "debtorId": ' . json_encode($id_debtor) . ',
            "number": "' . date('Y') . '/' . (int)$cart->id . '",
            "currency": ' . json_encode($currency->iso_code) . ',
            "downPayment": null,
            "discountRate": null,
            "financingStatus": null,
            "language": ' . json_encode(self::getLanguage($cart)) . ',
            "payment": {
                "term": "' . (int)$payment_term . 'INV"
            },
            "saleOrderNumber": null,
            "fields": [],
            ' . self::getInvoiceLinesJsonData($cart, $rounding_correction) . '
        }';
    }

    /**
     * Get lines of invoice in json format from cart
     * @param Cart $cart
     * @param float $rounding_correction rounding correction
     * @return string lines of invoice in json format
     * @throws \PrestaShop\PrestaShop\Core\Localization\Exception\LocalizationException
     */
    protected static function getInvoiceLinesJsonData(Cart $cart, $rounding_correction = 0)
    {
        $lines = '';
        foreach ($cart->getProducts() as $product) {
            $unit_price = $product['price_with_reduction_without_tax'];
            $quantity = $product['cart_quantity'];
            $discount_rate = 0;
            $discount_line = '';
            $tax_rate_line = '';
            if ($product['price_with_reduction_without_tax'] != $product['price_with_reduction']) {
                $tax_rate = KoalapayTaxRates::getTaxRateByCart($cart, $product);
                if (!$tax_rate) {
                    throw new Exception(self::l('No tax rate found for product (ID %s)'));
                }
                $tax_rate_line = '"taxRate": ' . json_encode($tax_rate) . ',';
            }
            if (isset($product['price_without_reduction_without_tax'])) {
                $price_without_reduction_without_tax = $product['price_without_reduction_without_tax'];
            } else {
                if (Configuration::get('PS_TAX_ADDRESS_TYPE') == 'id_address_invoice') {
                    $address_id = (int) $cart->id_address_invoice;
                } else {
                    $address_id = (int) $product['id_address_delivery'];
                }
                if (!Address::addressExists($address_id)) {
                    $address_id = null;
                }
                $specific_price_output = null;
                $price_without_reduction_without_tax = Product::getPriceStatic(
                    (int) $product['id_product'],
                    false,
                    isset($product['id_product_attribute']) ? (int) $product['id_product_attribute'] : null,
                    6,
                    null,
                    false,
                    false,
                    $product['quantity'],
                    false,
                    (int) $cart->id_customer ? (int) $cart->id_customer : null,
                    (int) $cart->id,
                    $address_id,
                    $specific_price_output,
                    true,
                    true,
                    Context::getContext(),
                    true,
                    $product['id_customization']
                );
            }
            if ($product['reduction_type'] == 'percentage') {
                $unit_price = $price_without_reduction_without_tax;
                $percent = 100 - ($product['price_with_reduction_without_tax'] / $unit_price * 100);
                $discount_rate = $percent / 100;
            } elseif ($product['reduction_type'] == 'amount' && $product['reduction_without_tax'] > 0) {
                $unit_price = $price_without_reduction_without_tax;
                $desc = sprintf(self::l('Discount -%s'), Tools::displayPrice($product['reduction']));
                $discount = round($product['reduction_without_tax'], 2);
                $discount_line .= ',{
                    "description": ' . json_encode($desc) . ',
                    "unitPrice": -' . (float)$discount . ',
                    "quantity": 1,
                    '.$tax_rate_line.'
                    "displayOrder": 1
                }';
            }
            if ($lines) {
                $lines .= ",\n";
            }
            $lines .= '{
                "description": ' . json_encode($product['name']) . ',
                "unitPrice": ' . (float)round($unit_price, 2) . ',
                "quantity": ' . (float)$quantity . ',
                "discount_rate": ' . (float)$discount_rate . ',
                '.$tax_rate_line.'
                "displayOrder": 1
            }';
            $lines .= $discount_line;
        }
        $discount = $cart->getOrderTotal(false, Cart::ONLY_DISCOUNTS);
        if ($discount > 0) {
            $tax_rate = KoalapayTaxRates::getDiscountTaxRateByCart($cart);
            $desc = self::l('Voucher(s)');
            $lines .= ',{
                "description": ' . json_encode($desc) . ',
                "unitPrice": -' . (float)$discount . ',
                "quantity": 1,
                "taxRate": ' . ($tax_rate ? json_encode($tax_rate) : 'null') . ',
                "displayOrder": 1
            }';
        }
        if ($rounding_correction != 0) {
            $desc = self::l('Rounding correction');
            $lines .= ',{
                "description": ' . json_encode($desc) . ',
                "unitPrice": ' . (float)$rounding_correction . ',
                "quantity": 1,
                "displayOrder": 1
            }';
        }
        $result = Hook::exec('actionKoalapayInvoiceLines', array(
            'cart' => $cart,
            'rounding_correction' => $rounding_correction,
            'json_lines' => $lines
        ));
        if ($result && json_decode($result)) {
            $lines = $result;
        }
        return '"lines": [' . $lines . ']';
    }

    /**
     * Get data in json format to send invoice
     * @param Cart $cart
     * @return string data in json format
     */
    protected static function getSendInvoiceJsonData(Cart $cart)
    {
        $customer = new Customer($cart->id_customer);
        $from = Configuration::get('PS_SHOP_EMAIL');
        $to = $customer->email;

        $subject = Configuration::get('KOALAPAY_EMAIL_SUBJECT', $cart->id_lang);
        $message = Configuration::get('KOALAPAY_EMAIL_MESSAGE', $cart->id_lang);

        $method = 'email';
        $post = '';
        if ($cart->getOrderTotal() >= 5000) {
            $method = 'postAndEmail';
            $post = ',
            "post": {
                "attachments": true,
                "envelope": "C4",
                "type": "EXPRESS"
            }';
        }

        return '{
            "method": ' . json_encode($method) . ',
            "email": {
                "copy": true,
                "receipt": true,
                "from": ' . json_encode($from) . ',
                "to": ' . json_encode($to) . ',
                "subject": ' . json_encode($subject) . ',
                "message": ' . json_encode($message) . '
            }' . $post . '
        }';
    }
}
