<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2021 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

require_once dirname(__FILE__) . '/vendor/autoload.php';
require_once dirname(__FILE__) . '/classes/Kpay.php';
require_once dirname(__FILE__) . '/classes/KoalapayTaxRates.php';
require_once dirname(__FILE__) . '/classes/KoalapayCustomerDebtor.php';
require_once dirname(__FILE__) . '/classes/KoalapayCartInvoice.php';
require_once dirname(__FILE__) . '/classes/KoalapayInvoiceStatus.php';
require_once dirname(__FILE__) . '/classes/KoalapayInvoiceFinancingStatus.php';

/*
    This comment is useful to the translator tools /!\ do not remove them

    $this->l('30-day payment solution for professionals and invoice factoring within 24h after client validation.')

    Kpay translations

    $this->l('Unknown Error')
    $this->l('Customer not found')
    $this->l('Debtor ID not found')
    $this->l('Invoice %s was not deleted via the api')
    $this->l('Invoice not found for Cart (ID %s)')
    $this->l('Discount -%s')
    $this->l('Rounding correction')
    $this->l('Invoice not found')
    $this->l('Unable to retrieve invoice')
    $this->l('No tax rate found for product (ID %s)')
    $this->l('Voucher(s)')
*/

/**
 * Class Koalapay
 */
class Koalapay extends PaymentModule
{
    protected $config_form = false;

    public function __construct()
    {
        $this->name = 'koalapay';
        $this->tab = 'payments_gateways';
        $this->version = '1.0.2';
        $this->currencies = true;
        $this->author = 'Stigmi';
        $this->need_instance = 1;

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;
        $this->module_key = 'd6fb3e41ce8f0525b58e6a815f92ef15';

        parent::__construct();

        $this->displayName = $this->l('Koalapay');
        $this->description = $this->l('30-days payment solution for professional buyers.');

        $this->confirmUninstall = $this->l('Are you sure?');

        $this->limited_currencies = array('CAD', 'CHF', 'CNY', 'EUR', 'GBP', 'JPY', 'NOK', 'SEK', 'USD');

        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
    }

    public function isPS17x()
    {
        return version_compare(_PS_VERSION_, '1.7', 'ge');
    }

    public function install()
    {
        if (extension_loaded('curl') == false) {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module');
            return false;
        }

        $groups_default_config  = '';
        $groups = Group::getGroups($this->context->language->id);
        foreach ($groups as $group) {
            $groups_default_config .= ','.$group['id_group'];
        }

        $order_states_default_config = '';
        foreach (OrderState::getOrderStates($this->context->language->id) as $order_state) {
            if ($order_state['template'] == 'shipped') {
                $order_states_default_config = (int)$order_state['id_order_state'];
            }
        }
        $groups_default_config = trim($groups_default_config, ',');

        $email_subject_default_config = array();
        $email_message_default_config = array();
        foreach (Language::getLanguages(false) as $lang) {
            $subject = $this->l('Your invoice', false, $lang['locale']);
            $msg = $this->l('Thank you for your purchase!', false, $lang['locale']);
            $email_subject_default_config[$lang['id_lang']] = $subject;
            $email_message_default_config[$lang['id_lang']] = $msg;
        }

        Configuration::updateValue('KOALAPAY_LOG_MODE', false);
        Configuration::updateValue('KOALAPAY_API_TOKEN', '');
        Configuration::updateValue('KOALAPAY_API_URL', 'https://connect.koalaboox.com/api');
        Configuration::updateValue('KOALAPAY_GROUPS', $groups_default_config);
        Configuration::updateValue('KOALAPAY_SEND_INVOICE_IMMEDIATELY', false);
        Configuration::updateValue('KOALAPAY_ORDER_STATES', $order_states_default_config);
        Configuration::updateValue('KOALAPAY_EMAIL_SUBJECT', $email_subject_default_config);
        Configuration::updateValue('KOALAPAY_EMAIL_MESSAGE', $email_message_default_config);
        Configuration::updateValue('KOALAPAY_CRON_SECURE_KEY', Tools::strtoupper(Tools::passwdGen(16)));

        include(dirname(__FILE__) . '/sql/install.php');

        $this->addStatus('KOALAPAY_OS_PAYMENT', array(
            'en' => 'Payment via Koalapay',
            'fr' => 'Paiement via Koalapay'
        ), array(
            'color' => '#90CF36',
            'template' => 'order_conf',
            'invoice' => 1,
            'paid' => 1,
            'send_email' => 1,
            'logable' => 1,
            'unremovable' => 1,
        ));
        $this->addStatus('KOALAPAY_OS_PAYMENT_PENDING', array(
            'en' => 'Payment via Koalapay - Invoice to send',
            'fr' => 'Paiement via Koalapay - Facture à envoyer'
        ), array(
            'color' => '#BFB500',
            'template' => '',
            'paid' => 0,
            'send_email' => 0,
            'logable' => 1,
            'unremovable' => 1,
        ));

        return parent::install() &&
            $this->installTabs() &&
            $this->registerHook('header') &&
            $this->registerHook('backOfficeHeader') &&
            $this->registerHook('payment') &&
            $this->registerHook('paymentReturn') &&
            $this->registerHook('paymentOptions') &&
            $this->registerHook('actionValidateOrder') &&
            $this->registerHook('actionCronJob') &&
            $this->registerHook('actionOrderHistoryAddAfter') &&
            $this->registerHook('actionOrderInvoiceAddBefore') &&
            $this->registerHook('actionOrderStatusPostUpdate') &&
            $this->registerHook('actionObjectCustomerDeleteAfter') &&
            $this->registerHook('actionSetInvoice') &&
            $this->registerHook('actionPDFInvoiceRender') &&
            $this->registerHook('displayBackOfficeOrderActions') &&
            $this->registerHook('displayPDFInvoice') &&
            $this->registerHook('actionObjectCartDeleteAfter');
    }

    public function uninstall()
    {
        Configuration::deleteByName('KOALAPAY_LOG_MODE');
        Configuration::deleteByName('KOALAPAY_API_TOKEN');
        Configuration::deleteByName('KOALAPAY_API_URL');
        Configuration::deleteByName('KOALAPAY_GROUPS');
        Configuration::deleteByName('KOALAPAY_SEND_INVOICE_IMMEDIATELY');
        Configuration::deleteByName('KOALAPAY_ORDER_STATES');
        Configuration::deleteByName('KOALAPAY_EMAIL_SUBJECT');
        Configuration::deleteByName('KOALAPAY_EMAIL_MESSAGE');
        Configuration::deleteByName('KOALAPAY_CRON_SECURE_KEY');

        include(dirname(__FILE__) . '/sql/uninstall.php');

        $this->uninstallTabs();

        return parent::uninstall();
    }

    /**
     * Adds new order state on install
     *
     * @param string $code
     * @param array $names
     * @param array $properties
     * @return boolean
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function addStatus($code, array $names = array(), array $properties = array())
    {
        if (Configuration::get($code)) {
            return true; // already exists
        }

        $order_state = new OrderState();
        foreach (Language::getLanguages(false) as $lang) {
            $iso_code = Tools::strtolower($lang['iso_code']);
            $name = isset($names[$iso_code]) ? $names[$iso_code] : $names['en'];
            $order_state->name[(int)$lang['id_lang']] = $name;
        }

        foreach ($properties as $property => $value) {
            $order_state->{$property} = $value;
        }

        $order_state->module_name = $this->name;
        $result = $order_state->add() && Validate::isLoadedObject($order_state);
        if ($result) {
            Configuration::updateValue($code, $order_state->id, false, false);
            $targets = array(
                _PS_IMG_DIR_ . DIRECTORY_SEPARATOR . 'os' . DIRECTORY_SEPARATOR . sprintf('%d.gif', $order_state->id),
                _PS_TMP_IMG_DIR_ . DIRECTORY_SEPARATOR . sprintf('order_state_mini_%d.gif', $order_state->id),
                _PS_TMP_IMG_DIR_ . DIRECTORY_SEPARATOR . sprintf(
                    'order_state_mini_%d_%d.gif',
                    $order_state->id,
                    Context::getContext()->shop->id
                )
            );
            foreach ($targets as $target) {
                $dir = dirname($target);
                if (!is_dir($dir)) {
                    mkdir($dir, 0777, true);
                }
                $source = dirname(__FILE__) . DIRECTORY_SEPARATOR . 'logo.gif';
                if (!copy($source, $target)) {
                    $this->_errors[] = sprintf($this->l('Unable to copy %s -> %s', $source, $target));
                }
            }
        }
        return $result;
    }

    /**
     * Installs tabs
     */
    public function installTabs()
    {
        $id_parent = (int)Tab::getIdFromClassName($this->isPS17x() ? 'AdminParentOrders' : 'AdminOrders');

        $tabs_to_add = array(
            'AdminKoalapayOrders' => 'Koalapay',
            'AdminKoalapayPdf' => 'Koalapay PDF'
        );
        foreach ($tabs_to_add as $class_name => $name) {
            if (Tab::getIdFromClassName($class_name)) {
                continue;
            }
            $tab = new Tab();
            $tab->id_parent = ($class_name == 'AdminKoalapayPdf') ? -1 : $id_parent;
            $tab->module = $this->name;
            $tab->class_name = $class_name;
            $tab->active = 1;
            foreach (Language::getLanguages(false) as $lang) {
                $tab->name[(int)$lang['id_lang']] = $name;
            }
            if (!$tab->save()) {
                return false;
            }
        }
        return true;
    }

    protected function uninstallTabs()
    {
        $result = true;
        $controllers = array('AdminKoalapayOrders');
        foreach ($controllers as $class_name) {
            $id = Tab::getIdFromClassName($class_name);
            if (!$id) {
                continue;
            }
            $tab = new Tab($id);
            if (!Validate::isLoadedObject($tab) || !$tab->delete()) {
                $result = false;
            }
        }
        return $result;
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */
        $tab_active = 'presentation';
        $form_messages_account = '';
        $form_messages_settings = '';
        if (((bool)Tools::isSubmit('submitKoalapayModuleAccount')) == true) {
            $form_messages_account = $this->postProcess('account');
            $tab_active = 'account';
        } elseif (((bool)Tools::isSubmit('submitKoalapayModuleSettings')) == true) {
            $form_messages_settings = $this->postProcess('settings');
            $tab_active = 'settings';
        }

        $account = false;
        $financing_status = false;
        $api_alive = Kpay::apiAlive();
        if (Configuration::get('KOALAPAY_API_TOKEN') && $api_alive === true) {
            $account = Kpay::getCurrentCustomer();
            if ($account) {
                $financing_status = Kpay::getFinancingStatus();
                $account->billing->address->addressLines = implode("<br/>", $account->billing->address->addressLines);
            }
        }

        $this->context->smarty->assign(array(
            'tab_active' => $tab_active,
            'module_dir' => $this->_path,
            'api_alive' => $api_alive,
            'api_url' => Configuration::get('KOALAPAY_API_URL'),
            'account' => $account,
            'financing_status' => $financing_status
        ));

        $output = $form_messages_settings;
        $output .= $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl');
        $output = str_replace('[[account_form]]', $form_messages_account . $this->renderForm('account'), $output);
        $output = str_replace('[[settings_form]]', $this->renderForm('settings'), $output);

        return $output;
    }

    /**
     * The form that will be displayed in the configuration.
     */
    protected function renderForm($form)
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitKoalapayModule'.Tools::ucfirst($form);
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues($form),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm($form)));
    }

    /**
     * The structure of configuration form.
     */
    protected function getConfigForm($form)
    {
        if ($form == 'account') {
            $inputs = array(
                array(
                    'type' => 'textarea',
                    'name' => 'KOALAPAY_API_TOKEN',
                    'label' => $this->l('API Token'),
                    'desc' => sprintf(
                        $this->l('Retrieve your personnal access token %s.'),
                        '<a href="https://connect.koalapay.com/home">' . $this->l('here') . '</a>'
                    ),
                    'required' => true,
                    ''
                ),
                array(
                    'type' => 'text',
                    'name' => 'KOALAPAY_API_URL',
                    'label' => $this->l('API Url'),
                    'desc' => $this->l('Only change this if necessary'),
                    'required' => true
                ),
            );
        } else {
            $groups = Group::getGroups($this->context->language->id);
            $order_states = OrderState::getOrderStates($this->context->language->id);
            foreach (OrderState::getOrderStates($this->context->language->id) as $key => $order_state) {
                if ($order_state['template'] == 'order_canceled'
                    || preg_match('#koalapay#i', $order_state['module_name'])) {
                    unset($order_states[$key]);
                }
            }
            $order_state_desc = $this->l('Order states that trigger the sending of the Koalapay invoice');
            $order_state_desc .= ' ' . $this->l('(if invoice has not already been sent)');
            $inputs = array(
                array(
                    'type' => 'free',
                    'name' => 'cron',
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Log mode'),
                    'name' => 'KOALAPAY_LOG_MODE',
                    'is_bool' => true,
                    'desc' => $this->l('Enable advanced logs?'),
                    'values' => array(
                        array(
                            'id' => 'active_on',
                            'value' => true,
                            'label' => $this->l('Enabled')
                        ),
                        array(
                            'id' => 'active_off',
                            'value' => false,
                            'label' => $this->l('Disabled')
                        )
                    )
                ),
                array(
                    'type' => 'group',
                    'label' => $this->l('Customer Groups'),
                    'desc' => $this->l('Enables Koalapay only if customers are in these groups'),
                    'name' => 'type',
                    'required' => true,
                    'values' => $groups
                ),
                array(
                    'type' => 'switch',
                    'label' => $this->l('Send the invoice when new order is created?'),
                    'name' => 'KOALAPAY_SEND_INVOICE_IMMEDIATELY',
                    'values' => array(
                        array(
                            'id' => 'active_on',
                            'value' => true,
                            'label' => $this->l('Yes')
                        ),
                        array(
                            'id' => 'active_off',
                            'value' => false,
                            'label' => $this->l('No')
                        )
                    )
                ),
                array(
                    'type' => 'order_states',
                    'label' => $this->l('Order States'),
                    'desc' => $order_state_desc,
                    'name' => 'type',
                    'required' => true,
                    'values' => $order_states
                )
            );
        }

        return array(
            'form' => array(
                'input' => $inputs,
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues($form)
    {
        if ($form == 'account') {
            return array(
                'KOALAPAY_API_URL' => Configuration::get('KOALAPAY_API_URL'),
                'KOALAPAY_API_TOKEN' => Configuration::get('KOALAPAY_API_TOKEN'),
            );
        }

        $cron = '';
        $cronjobs = Module::getInstanceByName('cronjobs');
        if (!$cronjobs || !$cronjobs->active) {
            $cron_url = $this->context->link->getModuleLink(
                $this->name,
                'cron',
                array('secure_key' => Configuration::get('KOALAPAY_CRON_SECURE_KEY'))
            );
            $this->smarty->assign('cron_url', $cron_url);
            $cron = $this->display(__FILE__, 'views/templates/admin/cron.tpl');
        }
        $values = array(
            'KOALAPAY_LOG_MODE' => (bool)Configuration::get('KOALAPAY_LOG_MODE'),
            'KOALAPAY_SEND_INVOICE_IMMEDIATELY' => (bool)Configuration::get('KOALAPAY_SEND_INVOICE_IMMEDIATELY'),
            'cron' => $cron
        );
        $groups = Group::getGroups($this->context->language->id);
        foreach ($groups as $group) {
            $values['groupBox_' . $group['id_group']] = in_array($group['id_group'], $this->getConfigGroups());
        }
        $order_states = OrderState::getOrderStates($this->context->language->id);
        foreach ($order_states as $order_state) {
            $state_box = (int)in_array($order_state['id_order_state'], $this->getConfigOrderStates());
            $values['stateBox_' . $order_state['id_order_state']] = $state_box;
        }
        return $values;
    }

    /**
     * Save form data.
     */
    protected function postProcess($form)
    {
        $form_values = $this->getConfigFormValues($form);
        foreach (array_keys($form_values) as $key) {
            if (preg_match('#^KOALAPAY_#i', $key)) {
                $lang_values = array();
                foreach (Language::getLanguages(false) as $lang) {
                    if (Tools::getIsset($key . '_' . $lang['id_lang'])) {
                        $lang_values[$lang['id_lang']] = Tools::getValue($key . '_' . $lang['id_lang']);
                    }
                }
                if (count($lang_values)) {
                    Configuration::updateValue($key, $lang_values, $key == 'KOALAPAY_EMAIL_MESSAGE');
                } else {
                    Configuration::updateValue($key, Tools::getValue($key));
                }
            }
        }

        $output = '';
        if ($form == 'account') {
            if (!Tools::getValue('KOALAPAY_API_TOKEN')) {
                $output .= $this->displayError($this->l('Please enter the API Token'));
            }
            if (!Validate::isUrl(Tools::getValue('KOALAPAY_API_URL'))) {
                $output .= $this->displayError($this->l('Please enter an API Url valid'));
            }
        }
        if ($form == 'settings') {
            Configuration::updateValue('KOALAPAY_GROUPS', '');
            if (is_array(Tools::getValue('groupBox'))) {
                Configuration::updateValue('KOALAPAY_GROUPS', implode(',', Tools::getValue('groupBox')));
            }

            Configuration::updateValue('KOALAPAY_ORDER_STATES', '');
            if (is_array(Tools::getValue('stateBox'))) {
                Configuration::updateValue('KOALAPAY_ORDER_STATES', implode(',', Tools::getValue('stateBox')));
            }
        }

        if ($output === '') {
            $output .= $this->displayConfirmation($this->l('Configuration updated'));
        }

        return $output;
    }

    /**
     * Add the CSS & JavaScript files
     */
    public function hookBackOfficeHeader()
    {
        if (Tools::getValue('configure') == $this->name) {
            $this->context->controller->addJquery();
            $this->context->controller->addJqueryUI('ui.slider');
            $this->context->controller->addCSS($this->_path . 'views/css/back.css');
        }
    }

    /**
     * Add the CSS & JavaScript files
     */
    public function hookHeader()
    {
        if (!$this->isPS17x()) {
            $this->context->controller->addCSS($this->_path . '/views/css/front.css');
        }
    }

    /**
     * Add payment option at the checkout in the front office (PS 1.6)
     */
    public function hookPayment($params)
    {
        $cart = $params['cart'];
        $id_currency = $cart->id_currency;
        $currency = new Currency((int)$id_currency);

        if (in_array($currency->iso_code, $this->limited_currencies) == false) {
            return false;
        }

        $result = $this->preparePayment($cart);
        if (!$result['show_method']) {
            return false;
        }

        $this->smarty->assign('module_dir', $this->_path);

        return $this->display(__FILE__, 'views/templates/hook/payment.tpl');
    }

    /**
     * This hook is used to display the order confirmation page.
     */
    public function hookPaymentReturn($params)
    {
        if ($this->active == false) {
            return;
        }

        if (isset($params['objOrder'])) {
            // backward compatibility
            $params['order'] = $params['objOrder'];
        }
        $order = $params['order'];

        if ($order->getCurrentOrderState()->id != Configuration::get('PS_OS_ERROR')) {
            $this->smarty->assign('status', 'ok');
        }

        $this->smarty->assign(array(
            'id_order' => $order->id,
            'reference' => $order->reference,
            'shop_name' => $this->context->shop->name,
            'module_dir', $this->_path,
            'total' => Tools::displayPrice(
                $order->getOrdersTotalPaid(),
                new Currency($order->id_currency),
                false
            ),
        ));

        return $this->display(__FILE__, 'views/templates/hook/payment_return.tpl');
    }

    /**
     * Return payment options available for PS 1.7+
     *
     * @param array Hook parameters
     *
     * @return array|null
     */
    public function hookPaymentOptions($params)
    {
        $cart = $params['cart'];

        if (!$this->checkCurrency($cart)) {
            return;
        }

        $result = $this->preparePayment($cart);
        if (!$result['show_method']) {
            return;
        }

        $infos = $this->fetch('module:koalapay/views/templates/hook/payment_infos.tpl');

        $option = new \PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $option->setCallToActionText($this->l('Pay in 30 days with Koalapay'))
            ->setLogo(Media::getMediaPath(_PS_MODULE_DIR_ . $this->name . '/logo.png'))
            ->setAdditionalInformation($infos)
            ->setAction($this->context->link->getModuleLink($this->name, 'validation', array(), true));

        return [
            $option
        ];
    }

    /**
     * Prepare payment, adds debtor, invoice and check eligibility
     * @param Cart $cart
     * @return infos
     *  [
     *      'show_method' => true|false if we show payment method
     *      'customer' => Customer|false
     *      'financing_status' => stdClass|false of current customer
     *      'debtor' => stdClass|false of debor
     *      'invoice' => stdClass|false|null of invoice
     *  ]
     * @throws Exception
     */
    public function preparePayment($cart)
    {
        $infos = array(
            'show_method' => false,
            'customer' => false,
            'financing_status' => false,
            'debtor' => false,
            'invoice' => false,
        );
        if (!$this->active) {
            return $infos;
        }
        if (!$cart->id_customer) {
            return $infos;
        }
        $customer = new Customer($cart->id_customer);
        if (!Validate::isLoadedObject($customer)) {
            return $infos;
        }
        $infos['customer'] = $customer;
        if (!$cart->id_address_invoice) {
            return $infos;
        }
        if (!$cart->id_carrier) {
            return $infos; // on payment step only
        }
        // Customer group check
        if (!in_array($customer->id_default_group, $this->getConfigGroups())) {
            $log = sprintf(
                $log = $this->l('Customer group (ID %s) is not eligible for Koalapay payment'),
                $customer->id_default_group
            );
            $this->log($log, 1, null, 'Cart', $cart->id);
            return $infos;
        }
        // Currency check
        $currency = new Currency((int)$cart->id_currency);
        if (in_array($currency->iso_code, $this->limited_currencies) == false) {
            $log = sprintf(
                $log = $this->l('Currency (%s) is not supported by Koalapay'),
                $currency->iso_code
            );
            $this->log($log, 1, null, 'Cart', $cart->id);
            return $infos;
        }
        // API alive?
        $api_alive = Kpay::apiAlive();
        if ($api_alive !== true) {
            $log = $this->l('API is down!?') . ' ' . $api_alive;
            $this->log($log, 3, null, 'Cart', $cart->id);
            return $infos;
        }
        // Financing status
        $financing_status = Kpay::getFinancingStatus();
        if (!$financing_status) {
            $log = $this->l('Unable to get financing status, check your API token');
            $this->log($log, 3, null, 'Cart', $cart->id);
            return $infos;
        }
        $infos['financing_status'] = $financing_status;
        if (!$financing_status->eligible) {
            $log = $this->l('Your account is not eligible');
            $this->log($log, 3, null, 'Cart', $cart->id);
            return $infos;
        }
        if (!$financing_status->active) {
            $log = $this->l('Your account is not active');
            $this->log($log, 3, null, 'Cart', $cart->id);
            return $infos;
        }
        if ($financing_status->limit < $cart->getOrderTotal(true)) {
            $log = sprintf(
                $this->l('Funding limit reached (total order: %s, limit: %s)'),
                $cart->getOrderTotal(true),
                $financing_status->limit
            );
            $this->log($log, 1, null, 'Cart', $cart->id);
            return $infos;
        }
        // Debtor
        try {
            $debtor = Kpay::getDebtor($customer);
            if (!isset($debtor->id)) {
                $debtor = Kpay::addDebtor($cart);
            } else {
                $debtor = Kpay::updateDebtor($debtor->id, $cart);
            }
            $infos['debtor'] = $debtor;
        } catch (Exception $e) {
            $log = sprintf($this->l('Unable to create/update debtor (Cart ID %d)'), $cart->id);
            $log .= sprintf($this->l(', %s'), $e->getMessage());
            $this->log($log, 2, null, 'Customer', $customer->id);
            return $infos;
        }
        // Invoice
        try {
            $invoice = Kpay::getInvoice($cart);
            if (!isset($invoice->id)) {
                $invoice = Kpay::addInvoice($cart, $debtor->id);
            } else {
                Kpay::deleteInvoice($invoice->id);
                $invoice = Kpay::getInvoice($cart);
                if (isset($invoice->id)) {
                    // koalapay invoice not deleted!?
                    $log = sprintf(
                        $this->l('No prestashop order for this koalapay invoice already sent (%s)???'),
                        $invoice->id
                    );
                    $this->log($log, 3, null, 'Cart', $cart->id);
                    $cart = $this->duplicateCart();
                } else {
                    KoalapayCartInvoice::deleteByCart($cart);
                }
                $invoice = Kpay::addInvoice($cart, $debtor->id);
            }
            $cart_total = round($cart->getOrderTotal(), 2);
            $koalapay_total = round($invoice->totals->total, 2);
            if ($cart_total != $koalapay_total) {
                $log = sprintf(
                    $this->l('Koalapay invoice amount (%s) and order amount (%s) are different'),
                    $koalapay_total,
                    $cart_total
                );
                $this->log($log, 4, null, 'Cart', $cart->id);
                return $infos;
            }
            $infos['invoice'] = $invoice;
        } catch (Exception $e) {
            $log = $this->l('Unable to create invoice');
            $log .= sprintf($this->l(', %s'), $e->getMessage());
            $this->log($log, 2, null, 'Cart', $cart->id);
            return $infos;
        }
        // Eligibility
        try {
            $eligibility = Kpay::checkEligibility($invoice->id);
            if ($eligibility->eligible !== true) {
                Kpay::deleteInvoice($invoice->id);
                $infos['invoice'] = null;
                $log = sprintf($this->l('Cart is not eligible (Invoice ID %s)'), $invoice->id);
                $log .= sprintf($this->l(', %s'), $eligibility->reason);
                $this->log($log, 1, null, 'Cart', $cart->id);
                return $infos;
            }
        } catch (Exception $e) {
            $log = sprintf($this->l('Unable to check eligibility (Invoice ID %s)'), $invoice->id);
            $log .= sprintf($this->l(', %s'), $e->getMessage());
            $this->log($log, 3, null, 'Cart', $cart->id);
            return $infos;
        }
        $infos['show_method'] = true;
        return $infos;
    }

    /**
     * Duplicate current cart
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    protected function duplicateCart()
    {
        $cart_products = $this->context->cart->getProducts();
        $this->context->cart->id = 0;
        $this->context->cart->add();
        foreach ($cart_products as $product) {
            $this->context->cart->updateQty(
                (int) $product['quantity'],
                (int) $product['id_product'],
                (int) $product['id_product_attribute']
            );
        }
        if ($this->context->cart->id) {
            $this->context->cookie->id_cart = (int) $this->context->cart->id;
            $this->context->cookie->write();
        }
        return $this->context->cart;
    }

    /**
     * This hooks is called when order is validated
     * @param array Hook parameters
     */
    public function hookActionValidateOrder($params)
    {
        $order = $params['order'];
        $cart = $params['cart'];
        try {
            if (Validate::isLoadedObject($order)) {
                if ($order->module != $this->name) {
                    if (!Validate::isLoadedObject($cart)) {
                        $cart = $order->id_cart;
                    }
                    if (!$cart) {
                        $this->log('No cart for order?', 3, null, 'Order', $order->id);
                    } else {
                        Kpay::deleteInvoiceByCart($cart);
                    }
                }
            } elseif (Validate::isLoadedObject($cart)) {
                $this->log('No order for cart?', 3, null, 'Cart', $cart->id);
                Kpay::deleteInvoiceByCart($cart);
            }
        } catch (Exception $e) {
            $object_type = null;
            $object_id = null;
            if (Validate::isLoadedObject($cart)) {
                $object_type = 'Cart';
                $object_id = $cart->id;
            }
            if (Validate::isLoadedObject($order)) {
                $object_type = 'Order';
                $object_id = $order->id;
            }
            $log = sprintf($this->l('Unable to delete invoice: %s'), $e->getMessage());
            $this->log($log, 3, null, $object_type, $object_id);
        }
    }

    /**
     * This hooks is called when new order history is created
     * @param array Hook parameters
     */
    public function hookActionOrderHistoryAddAfter($params)
    {
        $new_oh = $params['order_history'];
        if (in_array($new_oh->id_order_state, $this->getConfigOrderStates())) {
            try {
                $id_order = (int)$new_oh->id_order;
                $order = new Order($id_order);
                if ($order->module != $this->name) {
                    return;
                }
                if (!Validate::isLoadedObject($order)) {
                    throw new Exception(sprintf(
                        $this->l('Order not found (ID %s) for Order History (ID %s)'),
                        $id_order,
                        $new_oh->id
                    ));
                }
                $cart = new Cart($order->id_cart);
                if (!Validate::isLoadedObject($cart)) {
                    throw new Exception(sprintf(
                        $this->l('Cart not found (ID %s) for Order (ID %s)'),
                        $order->id_cart,
                        $id_order
                    ));
                }

                $updated_invoice = Kpay::sendInvoice($cart);

                $os = new OrderHistory();
                $os->id_order = $id_order;
                $os->changeIdOrderState(Configuration::get('KOALAPAY_OS_PAYMENT'), (int)$id_order);
                $os->save();

                $order_state = new OrderState($new_oh->id_order_state, $cart->id_lang);
                $log = sprintf(
                    $this->l('New Order State (%s), Koalapay Invoice (%s) was sent!'),
                    $order_state->name,
                    $updated_invoice->id
                );
                $this->log($log, 1, null, 'Order', $id_order);
            } catch (Exception $e) {
                $log = sprintf($this->l('Unable to send invoice: %s'), $e->getMessage());
                $this->log($log, 3, null, 'Order', $id_order);
            }
        }
    }

    /**
     * This hooks is called when order state for order change
     * @param array Hook parameters
     */
    public function hookActionOrderStatusPostUpdate($params)
    {
        $new_os = $params['newOrderStatus'];
        if ($new_os->id == Configuration::get('KOALAPAY_OS_PAYMENT')) {
            try {
                $id_order = (int)$params['id_order'];
                $order = new Order($id_order);
                if (!Validate::isLoadedObject($order)) {
                    throw new Exception(sprintf(
                        $this->l('Order not found (ID %s)'),
                        $id_order
                    ));
                }
                $cart_invoice = KoalapayCartInvoice::getCartInvoice($order);
                if (!Validate::isLoadedObject($cart_invoice)) {
                    throw new Exception('No Koalapay Payment for this order!');
                }
                if (!$cart_invoice->is_sent) {
                    Kpay::sendInvoice(new Cart($cart_invoice->id_cart));
                    $log = sprintf(
                        $this->l('New Payment Status, Koalapay Invoice (%s) was sent!'),
                        $cart_invoice->id_invoice
                    );
                    $this->log($log, 1, null, 'Order', $id_order);
                }
                if ($order->getTotalPaid() === 0) {
                    $order->addOrderPayment($cart_invoice->total);
                }
            } catch (Exception $e) {
                $log = sprintf($this->l('Unable to create order payment: %s'), $e->getMessage());
                $this->log($log, 2, null, 'Order', $id_order);
            }
        }
    }

    /**
     * This hooks is called to show order actions in backend
     * @param $params Hook parameters
     * @return string order actions
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     * @throws SmartyException
     */
    public function hookDisplayBackOfficeOrderActions($params)
    {
        $output = '';
        $order = new Order($params['id_order']);
        if (Validate::isLoadedObject($order)) {
            $cart_invoice = KoalapayCartInvoice::getCartInvoice($order);
            $show_invoice_pdf = false;
            if (Validate::isLoadedObject($cart_invoice)) {
                $show_invoice_pdf = !in_array($cart_invoice->status, array('draft', 'canceled', ''));
            }
            $this->context->smarty->assign(array(
                'order' => $order,
                'show_invoice_pdf' => $show_invoice_pdf
            ));
            $output = $this->context->smarty->fetch($this->local_path . 'views/templates/hook/order_actions.tpl');
        }
        return $output;
    }

    /**
     * This hooks is called when pdf invoice is generated
     * @param array Hook parameters
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function hookActionPDFInvoiceRender($params)
    {
        if ($this->context->controller instanceof AdminKoalapayPdfController
            || $this->context->controller instanceof KoalapayPdfModuleFrontController) {
            return;
        }
        $order_invoice_list = $params['order_invoice_list'];
        if (count($order_invoice_list) == 1) {
            foreach ($order_invoice_list as $order_invoice) {
                $order = new Order($order_invoice->id_order);
                if (!Validate::isLoadedObject($order)) {
                    $log = sprintf(
                        $this->l('Order not found (ID %s) for Order Invoice (ID %s)'),
                        $order_invoice->id_order,
                        $order_invoice->id
                    );
                    $this->log($log, 3, null, 'Order', $order_invoice->id_order);
                }
                $cart_invoice = KoalapayCartInvoice::getCartInvoice($order);
                if (Validate::isLoadedObject($cart_invoice)) {
                    // koaloboox invoice
                    $params = array('id_order' => $order->id);
                    if ($this->context->controller instanceof AdminController) {
                        Tools::redirectAdmin($this->getAdminLink(
                            'AdminKoalapayPdf',
                            true,
                            array(),
                            $params
                        ));
                    } else {
                        Tools::redirect($this->context->link->getModuleLink('koalapay', 'pdf', $params));
                    }
                }
            }
        }
    }

    /**
     * This hooks is used to display a warning message at the bottom of the prestashop pdf invoice
     * @param array $param Order Invoice
     * @return false|string warning message
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     * @throws SmartyException
     */
    public function hookDisplayPDFInvoice($param)
    {
        $order_invoice = $param['object'];
        $order = new Order((int) $order_invoice->id_order);
        $cart_invoice = KoalapayCartInvoice::getCartInvoice($order);
        if (Validate::isLoadedObject($cart_invoice)) {
            $params = array('id_order' => $order->id);
            if ($this->context->controller instanceof AdminController) {
                $pdf_link = $this->getAdminLink(
                    'AdminKoalapayPdf',
                    true,
                    array(),
                    $params
                );
            } else {
                $pdf_link = $this->context->link->getModuleLink('koalapay', 'pdf', $params);
            }
            $this->context->smarty->assign(array(
                'pdf_link' => $pdf_link
            ));
            return $this->context->smarty->fetch($this->local_path . 'views/templates/hook/pdf_invoice.tpl');
        }
        return '';
    }

    /**
     * This hooks is called when order invoice is created
     * @param array Hook parameters
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function hookActionOrderInvoiceAddBefore($params)
    {
        $order_invoice = $params['object'];
        $order = new Order($order_invoice->id_order);
        if (!Validate::isLoadedObject($order)) {
            $log = sprintf(
                $this->l('Order not found (ID %s) for Order Invoice (ID %s)'),
                $order_invoice->id_order,
                $order_invoice->id
            );
            $this->log($log, 3, null, 'Order', $order_invoice->id_order);
            return;
        }
        $cart_invoice = KoalapayCartInvoice::getCartInvoice($order);
        if (Validate::isLoadedObject($cart_invoice)) {
            // no prestashop invoice
            $params['object']->number = 0;
        }
    }

    /**
     * Print PDF koalapay invoice
     * @param int $id_order ID Order
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function renderPDF($id_order)
    {
        $order = new Order((int)$id_order);
        if (!Validate::isLoadedObject($order)) {
            $msg = $this->l('Order not found');
            $this->log($msg, 2, null, 'Order', $id_order);
            die($msg);
        }

        if ($this->context->controller instanceof FrontController
            && $order->id_customer != $this->context->customer->id) {
            $msg = sprintf($this->l('Order not found for Customer (ID %s)'), $this->context->customer->id);
            $this->log($msg, 2, null, 'Order', $id_order);
            die($msg);
        }

        $cart_invoice = KoalapayCartInvoice::getCartInvoice($order);
        if (!Validate::isLoadedObject($cart_invoice)) {
            $msg = sprintf($this->l('No Koalapay invoice found for Order (ID %d)'), $id_order);
            $this->log($msg, 2, null, 'Order', $id_order);
            die($msg);
        }

        try {
            $pdf = Kpay::getPdfInvoice($cart_invoice->id_invoice);
            $filename = "koalapay-invoice#{$cart_invoice->invoice_number}.pdf";
            header("Content-type: application/pdf");
            header("Content-Disposition:attachment;filename=$filename");
            die($pdf);
        } catch (Exception $e) {
            $msg = sprintf(
                $this->l('An error occurred when generating pdf invoice %s: %s'),
                $cart_invoice->id_invoice,
                $e->getMessage()
            );
            $this->log($msg, 3, null, 'Order', $id_order);
            die($msg);
        }
    }

    /**
     * This hooks is called when invoice is created
     * @param $params
     * @return int invoice number
     */
    public function hookActionSetInvoice($params)
    {
        if ((int)Configuration::get('PS_INVOICE')) {
            $order = $params['Order'];
            if ($order->module == 'koalapay') {
                $cart_invoice = KoalapayCartInvoice::getCartInvoice($order);
                if (Validate::isLoadedObject($cart_invoice)) {
                    // no prestashop invoice : pdf invoice should not be sent
                    $order->invoice_number = 0;
                }
            }
        }
        return $order->invoice_number;
    }

    /**
     * This hooks is called when customer is deleted
     * @param array Hook parameters
     */
    public function hookActionObjectCustomerDeleteAfter($params)
    {
        KoalapayCustomerDebtor::deleteByCustomer($params['object']->id);
    }

    /**
     * This hooks is called when cart is deleted
     * @param array Hook parameters
     */
    public function hookActionObjectCartDeleteAfter($params)
    {
        $cart = $params['object'];
        if (Validate::isLoadedObject($cart) && !$cart->id_order) {
            try {
                Kpay::deleteInvoiceByCart($cart->id);
            } catch (Exception $e) {
                $log = sprintf($this->l('Unable to delete invoice: %s'), $e->getMessage());
                $this->log($log, 3, null, 'Cart', $cart->id);
            }
        }
    }

    /**
     * Delete abandonned invoices (7 days back)
     * Refresh invoices status (60 days back)
     */
    public function hookActionCronJob()
    {
        $invoices = KoalapayCartInvoice::getAbandonedInvoices(7);
        foreach ($invoices as $invoice) {
            $id_cart_invoice = $invoice['id_cart_invoice'];
            $id_cart = $invoice['id_cart'];
            $id_invoice = $invoice['id_invoice'];
            try {
                if (Kpay::deleteInvoice($id_invoice) === false) {
                    $err = sprintf(self::l('Invoice %s was not deleted via the api'), $id_invoice);
                    throw new Exception($err);
                }
                $k_invoice = new KoalapayCartInvoice($id_cart_invoice);
                $k_invoice->delete();
                $log = sprintf($this->l('Cron - Invoice %s successfully deleted'), $id_invoice);
                $this->log($log, 1, null, 'Cart', $id_cart);
            } catch (Exception $e) {
                $log = sprintf($this->l('Cron - Invoice %s unsuccessfully deleted'), $id_invoice);
                $log .= sprintf($this->l(', %s'), $e->getMessage());
                $this->log($log, 3, null, 'KoalapayCartInvoice', $id_cart_invoice);
            }
        }
        $invoices = KoalapayCartInvoice::getInvoicesToRefresh(60);
        foreach ($invoices as $invoice) {
            $id_cart_invoice = $invoice['id_cart_invoice'];
            $id_cart = $invoice['id_cart'];
            $id_invoice = $invoice['id_invoice'];
            try {
                $cart = new Cart($invoice['id_cart']);
                if (!Validate::isLoadedObject($cart)) {
                    $log = sprintf($this->l('Cart not found (ID %s) for Invoice %s'), $id_cart, $id_invoice);
                    $this->log($log, 1, null, 'KoalapayCartInvoice', $id_cart_invoice);
                }
                Kpay::refreshStatus($cart);
            } catch (Exception $e) {
                $log = sprintf($this->l('Cron - Status of invoice %s unsuccessfully refreshed'), $id_invoice);
                $log .= sprintf($this->l(', %s'), $e->getMessage());
                $this->log($log, 3, null, 'KoalapayCartInvoice', $id_cart_invoice);
            }
        }
    }

    /**
     * Get cron frequency (each day at 01:00)
     * @return int[] cron frequency
     */
    public function getCronFrequency()
    {
        return array(
            'hour' => 1,
            'day' => -1,
            'month' => -1,
            'day_of_week' => -1
        );
    }

    /**
     * @param $cart
     * @return bool
     */
    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);
        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Get customer groups that enable kolalaboox
     * @return array $groups
     */
    public function getConfigGroups()
    {
        $groups = explode(',', Configuration::get('KOALAPAY_GROUPS'));
        if (!$groups) {
            return array();
        }
        foreach ($groups as $i => $group) {
            $groups[$i] = trim($group);
            if ('' === $groups[$i]) {
                unset($groups[$i]);
            }
        }
        return $groups;
    }

    /**
     * Get order states that send kolalaboox invoice
     * @return array $order_states
     */
    public function getConfigOrderStates()
    {
        $order_states = explode(',', Configuration::get('KOALAPAY_ORDER_STATES'));
        if (!$order_states) {
            return array();
        }
        foreach ($order_states as $i => $order_state) {
            $order_states[$i] = trim($order_state);
            if ('' === $order_states[$i]) {
                unset($order_states[$i]);
            }
        }
        return $order_states;
    }

    /**
     * Log only in dev mode
     * @param $msg log message
     * @param int $severity 1 to 4
     * @param null $errorCode
     * @param null $objectType
     * @param null $objectId
     */
    public function log($msg, $severity = 2, $errorCode = null, $objectType = null, $objectId = null)
    {
        $msg = 'KOALAPAY: [' .$_SERVER['REMOTE_ADDR'] . '] ' . $msg;
        if (Configuration::get('KOALAPAY_LOG_MODE') || $severity > 2) {
            PrestaShopLogger::addLog($msg, $severity, $errorCode, $objectType, $objectId);
        }
    }


    /**
     * Surround $link->getAdminLink() method for backward compatibility
     * @param $controller
     * @param bool $withToken
     * @param array $sfRouteParams
     * @param array $params
     */
    public function getAdminLink($controller, $withToken = true, $sfRouteParams = array(), $params = array())
    {
        if (!$this->isPS17x()) {
            $link = $this->context->link->getAdminLink($controller, $withToken);
            $query = http_build_query($params, '', '&');
            if (preg_match('#\?#', $link)) {
                $link .= '&';
            } else {
                $link .= '?';
            }
            return $link.$query;
        }

        return $this->context->link->getAdminLink($controller, $withToken, $sfRouteParams, $params);
    }
}
